// date-time-parser.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* WHITESPACE_CHARS = NULL;

Cons* TIME_DIVIDERS = NULL;

Cons* DATE_DIVIDERS = NULL;

// If TRUE, an error will be signalled if parse-date-time is unable
//    to determine the time/date format of the string.
DEFINE_STELLA_SPECIAL(oERROR_ON_MISMATCHo, boolean , FALSE);

Object* hashlistExpander(Cons* list, HashTable* table) {
  return (listO(7, SYM_DATE_TIME_PARSER_STELLA_FOREACH, SYM_DATE_TIME_PARSER_STELLA_ITEM, SYM_DATE_TIME_PARSER_STELLA_IN, listO(3, SYM_DATE_TIME_PARSER_STELLA_CAST, list, cons(listO(4, SYM_DATE_TIME_PARSER_STELLA_CONS, SYM_DATE_TIME_PARSER_STELLA_OF, SYM_DATE_TIME_PARSER_STELLA_CONS, NIL), NIL)), SYM_DATE_TIME_PARSER_STELLA_DO, listO(3, SYM_DATE_TIME_PARSER_STELLA_INSERT_AT, table, listO(3, listO(3, SYM_DATE_TIME_PARSER_STELLA_FIRST, SYM_DATE_TIME_PARSER_STELLA_ITEM, NIL), listO(3, SYM_DATE_TIME_PARSER_STELLA_SECOND, SYM_DATE_TIME_PARSER_STELLA_ITEM, NIL), NIL)), NIL));
}

DEFINE_STELLA_SPECIAL(oWEEKDAY_STRINGSo, StringHashTable* , NULL);

DEFINE_STELLA_SPECIAL(oMONTH_STRINGSo, StringToIntegerHashTable* , NULL);

DEFINE_STELLA_SPECIAL(oZONE_STRINGSo, StringHashTable* , NULL);

DEFINE_STELLA_SPECIAL(oSPECIAL_STRINGSo, StringHashTable* , NULL);

DEFINE_STELLA_SPECIAL(oSPECIAL_SYMBOLSo, Cons* , NULL);

DEFINE_STELLA_SPECIAL(oDATE_TIME_PATTERNSo, Cons* , NULL);

void yesterday(DecodedDateTime* parsedValues) {
  today(parsedValues);
  { int jd = computeJulianDay(parsedValues->dateTimeYear, parsedValues->dateTimeMonth, parsedValues->dateTimeDay);

    { int year = NULL_INTEGER;
      int month = NULL_INTEGER;
      int day = NULL_INTEGER;
      Keyword* dow = NULL;

      year = computeCalendarDate(jd - 1, month, day, dow);
      parsedValues->dateTimeDay = day;
      parsedValues->dateTimeMonth = month;
      parsedValues->dateTimeYear = year;
      parsedValues->dateTimeDow = dow;
    }
  }
}

void today(DecodedDateTime* parsedValues) {
  { int year = NULL_INTEGER;
    int month = NULL_INTEGER;
    int day = NULL_INTEGER;
    Keyword* dow = NULL;

    year = makeCurrentDateTime()->getCalendarDate(parsedValues->dateTimeZone, month, day, dow);
    parsedValues->dateTimeDay = day;
    parsedValues->dateTimeMonth = month;
    parsedValues->dateTimeYear = year;
    parsedValues->dateTimeDow = dow;
  }
}

void tomorrow(DecodedDateTime* parsedValues) {
  today(parsedValues);
  { int jd = computeJulianDay(parsedValues->dateTimeYear, parsedValues->dateTimeMonth, parsedValues->dateTimeDay);

    { int year = NULL_INTEGER;
      int month = NULL_INTEGER;
      int day = NULL_INTEGER;
      Keyword* dow = NULL;

      year = computeCalendarDate(jd + 1, month, day, dow);
      parsedValues->dateTimeDay = day;
      parsedValues->dateTimeMonth = month;
      parsedValues->dateTimeYear = year;
      parsedValues->dateTimeDow = dow;
    }
  }
}

void now(DecodedDateTime* parsedValues) {
  { int hour = NULL_INTEGER;
    int minute = NULL_INTEGER;
    int second = NULL_INTEGER;
    int millis = NULL_INTEGER;

    hour = makeCurrentDateTime()->getTime(parsedValues->dateTimeZone, minute, second, millis);
    parsedValues->dateTimeMillis = millis;
    parsedValues->dateTimeSecond = second;
    parsedValues->dateTimeMinute = minute;
    parsedValues->dateTimeHour = hour;
  }
}

Symbol* amPm(Object* string) {
  if (subtypeOfStringP(safePrimaryType(string))) {
    { Object* string000 = string;
      StringWrapper* string = ((StringWrapper*)(string000));

      if (stringEqlP(string->wrapperValue, "am")) {
        return (SYM_DATE_TIME_PARSER_STELLA_AM);
      }
      else if (stringEqlP(string->wrapperValue, "pm")) {
        return (SYM_DATE_TIME_PARSER_STELLA_PM);
      }
      else {
        return (NULL);
      }
    }
  }
  else {
    return (NULL);
  }
}

Symbol* noonMidn(Object* string) {
  if (subtypeOfStringP(safePrimaryType(string))) {
    { Object* string000 = string;
      StringWrapper* string = ((StringWrapper*)(string000));

      if (stringEqlP(string->wrapperValue, "noon")) {
        return (SYM_DATE_TIME_PARSER_STELLA_NOON);
      }
      else if (stringEqlP(string->wrapperValue, "midnight")) {
        return (SYM_DATE_TIME_PARSER_STELLA_MIDN);
      }
      else {
        return (NULL);
      }
    }
  }
  else {
    return (NULL);
  }
}

Keyword* weekday(Object* thing) {
  if (subtypeOfStringP(safePrimaryType(thing))) {
    { Object* thing000 = thing;
      StringWrapper* thing = ((StringWrapper*)(thing000));

      return (((Keyword*)(oWEEKDAY_STRINGSo.get()->lookup(thing->wrapperValue))));
    }
  }
  else {
    return (NULL);
  }
}

IntegerWrapper* month(Object* thing) {
  { Surrogate* testValue000 = safePrimaryType(thing);

    if (subtypeOfStringP(testValue000)) {
      { Object* thing000 = thing;
        StringWrapper* thing = ((StringWrapper*)(thing000));

        { int monthNumber = oMONTH_STRINGSo.get()->lookup(thing->wrapperValue);

          if (monthNumber != NULL_INTEGER) {
            return (wrapInteger(monthNumber));
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else if (subtypeOfIntegerP(testValue000)) {
      { Object* thing001 = thing;
        IntegerWrapper* thing = ((IntegerWrapper*)(thing001));

        if ((1 <= thing->wrapperValue) &&
            (thing->wrapperValue <= 12)) {
          return (thing);
        }
        else {
          return (NULL);
        }
      }
    }
    else {
      return (NULL);
    }
  }
}

FloatWrapper* zone(Object* thing) {
  { Surrogate* testValue000 = safePrimaryType(thing);

    if (subtypeOfStringP(testValue000)) {
      { Object* thing000 = thing;
        StringWrapper* thing = ((StringWrapper*)(thing000));

        { FloatWrapper* zoneNumber = ((FloatWrapper*)(oZONE_STRINGSo.get()->lookup(thing->wrapperValue)));

          if (((boolean)(zoneNumber))) {
            return (zoneNumber);
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else if (subtypeOfFloatP(testValue000)) {
      { Object* thing001 = thing;
        FloatWrapper* thing = ((FloatWrapper*)(thing001));

        if ((-12.0 <= thing->wrapperValue) &&
            (thing->wrapperValue <= 14.0)) {
          return (thing);
        }
        else {
          return (NULL);
        }
      }
    }
    else if (subtypeOfIntegerP(testValue000)) {
      { Object* thing002 = thing;
        IntegerWrapper* thing = ((IntegerWrapper*)(thing002));

        { int offset = thing->wrapperValue;

          if ((-12 <= offset) &&
              (offset <= 14)) {
            return (wrapFloat(((double)(offset))));
          }
          else if ((-1200 <= offset) &&
              (offset <= 1400)) {
            return (wrapFloat(floor_(offset / 100.0) + ((offset % 100) / 60.0)));
          }
          else {
            return (NULL);
          }
        }
      }
    }
    else {
      return (NULL);
    }
  }
}

Symbol* specialp(Object* string) {
  if (subtypeOfStringP(safePrimaryType(string))) {
    { Object* string000 = string;
      StringWrapper* string = ((StringWrapper*)(string000));

      return (((Symbol*)(oSPECIAL_STRINGSo.get()->lookup(string->wrapperValue))));
    }
  }
  else {
    return (NULL);
  }
}

BooleanWrapper* secondp(Object* number) {
  { Surrogate* testValue000 = safePrimaryType(number);

    if (subtypeOfIntegerP(testValue000)) {
      { Object* number000 = number;
        IntegerWrapper* number = ((IntegerWrapper*)(number000));

        return ((((0 <= number->wrapperValue) &&
            (number->wrapperValue <= 59)) ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
    }
    else if (subtypeOfFloatP(testValue000)) {
      { Object* number001 = number;
        FloatWrapper* number = ((FloatWrapper*)(number001));

        return ((((0.0 <= number->wrapperValue) &&
            (number->wrapperValue < 60.0)) ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
    }
    else {
      return (FALSE_WRAPPER);
    }
  }
}

BooleanWrapper* minute(Object* number) {
  if (subtypeOfIntegerP(safePrimaryType(number))) {
    { Object* number000 = number;
      IntegerWrapper* number = ((IntegerWrapper*)(number000));

      return ((((0 <= number->wrapperValue) &&
          (number->wrapperValue <= 59)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

BooleanWrapper* hour(Object* number) {
  if (subtypeOfIntegerP(safePrimaryType(number))) {
    { Object* number000 = number;
      IntegerWrapper* number = ((IntegerWrapper*)(number000));

      return ((((0 <= number->wrapperValue) &&
          (number->wrapperValue <= 23)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

BooleanWrapper* day(Object* number) {
  if (subtypeOfIntegerP(safePrimaryType(number))) {
    { Object* number000 = number;
      IntegerWrapper* number = ((IntegerWrapper*)(number000));

      return ((((1 <= number->wrapperValue) &&
          (number->wrapperValue <= 31)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

BooleanWrapper* year(Object* number) {
  if (subtypeOfIntegerP(safePrimaryType(number))) {
    { Object* number000 = number;
      IntegerWrapper* number = ((IntegerWrapper*)(number000));

      return ((((-4713 <= number->wrapperValue) ||
          (number->wrapperValue <= 5000)) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

BooleanWrapper* timeDivider(Object* thecharacter) {
  if (subtypeOfCharacterP(safePrimaryType(thecharacter))) {
    { Object* thecharacter000 = thecharacter;
      CharacterWrapper* thecharacter = ((CharacterWrapper*)(thecharacter000));

      return ((TIME_DIVIDERS->memberP(thecharacter) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

BooleanWrapper* dateDivider(Object* thecharacter) {
  if (subtypeOfCharacterP(safePrimaryType(thecharacter))) {
    { Object* thecharacter000 = thecharacter;
      CharacterWrapper* thecharacter = ((CharacterWrapper*)(thecharacter000));

      return ((DATE_DIVIDERS->memberP(thecharacter) ? TRUE_WRAPPER : FALSE_WRAPPER));
    }
  }
  else {
    return (FALSE_WRAPPER);
  }
}

Cons* matchSubstring(char* substring) {
  { char* teststring = stringDowncase(substring);
    Object* testValue = NULL;

    if (((boolean)(((IntegerWrapper*)(testValue = month(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_MONTH, testValue));
    }
    if (((boolean)(((Keyword*)(testValue = weekday(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_WEEKDAY, testValue));
    }
    if (((boolean)(((Symbol*)(testValue = amPm(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_AM_PM, testValue));
    }
    if (((boolean)(((Symbol*)(testValue = noonMidn(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN, testValue));
    }
    if (((boolean)(((FloatWrapper*)(testValue = zone(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_ZONE, testValue));
    }
    if (((boolean)(((Symbol*)(testValue = specialp(wrapString(teststring))))))) {
      return (consList(2, SYM_DATE_TIME_PARSER_STELLA_SPECIAL, testValue));
    }
    if (oERROR_ON_MISMATCHo.get()) {
      { OutputStringStream* stream000 = newOutputStringStream();

        *(stream000->nativeStream) << "\"~A\" is not a recognized word or abbreviation." << "`" << substring << "'";
        throw *newStellaException(stream000->theStringReader());
      }
    }
    else {
      return (NIL);
    }
  }
}

int digitValue(char chaR) {
  return ((int)(unsigned char) chaR - (int)(unsigned char) '0');
}

Cons* decomposeDateTimeString(char* string, int start, int end) {
  { int stringIndex = start;
    boolean nextNegativeP = FALSE;
    Cons* partsList = NIL;
    char nextChar = NULL_CHARACTER;
    char prevChar = NULL_CHARACTER;

    if (end == NULL_INTEGER) {
      end = strlen(string);
    }
    while (stringIndex < end) {
      nextChar = string[stringIndex];
      prevChar = ((stringIndex == start) ? NULL_CHARACTER : string[(stringIndex - 1)]);
      if (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) nextChar] == KWD_DATE_TIME_PARSER_LETTER) {
        { int scanIndex = NULL_INTEGER;
          int iter000 = stringIndex + 1;

          while (TRUE) {
            scanIndex = iter000;
            iter000 = iter000 + 1;
            if ((scanIndex == end) ||
                (!(oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (string[scanIndex])] == KWD_DATE_TIME_PARSER_LETTER))) {
              { Cons* matchSymbol = matchSubstring(stringSubsequence(string, stringIndex, scanIndex));

                if (matchSymbol == NIL) {
                  return (NIL);
                }
                else {
                  partsList = cons(matchSymbol, partsList);
                }
              }
              stringIndex = scanIndex;
              break;
            }
          }
        }
      }
      else if (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) nextChar] == KWD_DATE_TIME_PARSER_DIGIT) {
        { int numericValue = digitValue(nextChar);
          int fractionalValue = 0;
          double fractionalDivisor = 1.0;
          boolean inFractionP = FALSE;

          { int scanIndex = NULL_INTEGER;
            int iter001 = stringIndex + 1;

            while (TRUE) {
              scanIndex = iter001;
              iter001 = iter001 + 1;
              if ((scanIndex == end) ||
                  (!((oCHARACTER_TYPE_TABLEo[(int)(unsigned char) (string[scanIndex])] == KWD_DATE_TIME_PARSER_DIGIT) ||
                  ((!inFractionP) &&
                   (string[scanIndex] == '.'))))) {
                if (nextNegativeP) {
                  nextNegativeP = FALSE;
                  numericValue = 0 - numericValue;
                  fractionalValue = 0 - fractionalValue;
                }
                if (inFractionP) {
                  partsList = cons(wrapFloat(numericValue + (fractionalValue / fractionalDivisor)), partsList);
                }
                else {
                  partsList = cons(wrapInteger(numericValue), partsList);
                }
                stringIndex = scanIndex;
                break;
              }
              if (inFractionP) {
                fractionalValue = (fractionalValue * 10) + digitValue(string[scanIndex]);
                fractionalDivisor = fractionalDivisor * 10.0;
              }
              else if (string[scanIndex] == '.') {
                inFractionP = TRUE;
              }
              else {
                numericValue = (numericValue * 10) + digitValue(string[scanIndex]);
              }
            }
          }
        }
      }
      else if ((nextChar == '-') &&
          ((prevChar == NULL_CHARACTER) ||
           (WHITESPACE_CHARS->memberP(wrapCharacter(prevChar)) ||
            DATE_DIVIDERS->memberP(wrapCharacter(prevChar))))) {
        nextNegativeP = TRUE;
        stringIndex = stringIndex + 1;
      }
      else if (TIME_DIVIDERS->memberP(wrapCharacter(nextChar))) {
        partsList = cons(consList(2, SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER, wrapCharacter(nextChar)), partsList);
        stringIndex = stringIndex + 1;
      }
      else if (DATE_DIVIDERS->memberP(wrapCharacter(nextChar))) {
        partsList = cons(consList(2, SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER, wrapCharacter(nextChar)), partsList);
        stringIndex = stringIndex + 1;
      }
      else if (WHITESPACE_CHARS->memberP(wrapCharacter(nextChar))) {
        stringIndex = stringIndex + 1;
      }
      else if (nextChar == '+') {
        partsList = cons(consList(2, SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER, wrapCharacter(nextChar)), partsList);
        stringIndex = stringIndex + 1;
      }
      else if (nextChar == '(') {
        while ((!(stringIndex == end)) &&
            (!(string[stringIndex] == ')'))) {
          stringIndex = stringIndex + 1;
        }
      }
      else {
        if (oERROR_ON_MISMATCHo.get()) {
          { OutputStringStream* stream000 = newOutputStringStream();

            *(stream000->nativeStream) << ">>> Bogus character at position " << "`" << stringIndex << "'" << " in \"" << "`" << string << "'" << "\"";
            throw *newStellaException(stream000->theStringReader());
          }
        }
        else {
          return (NIL);
        }
      }
    }
    return (partsList->reverse());
  }
}

Cons* matchPatternElement(Symbol* patternElement, Object* datumElement) {
  if (isaP(datumElement, SGT_DATE_TIME_PARSER_STELLA_CONS)) {
    { Object* datumType = ((((Cons*)(datumElement))->value == SYM_DATE_TIME_PARSER_STELLA_SPECIAL) ? ((Cons*)(datumElement))->rest->value : ((Cons*)(datumElement))->value);

      if (datumType == patternElement) {
        return (((Cons*)(datumElement)));
      }
      else {
        return (NIL);
      }
    }
  }
  else if (oSPECIAL_SYMBOLSo.get()->memberP(patternElement)) {
    return (NIL);
  }
  else {
    { cpp_function_code function = lookupFunction(patternElement)->functionCode;
      Object* match = ((Object*  (*) (Object*))function)(datumElement);

      if (subtypeOfBooleanP(safePrimaryType(match))) {
        { Object* match000 = match;
          BooleanWrapper* match = ((BooleanWrapper*)(match000));

          if (coerceWrappedBooleanToBoolean(match)) {
            return (consList(2, patternElement, datumElement));
          }
          else {
            return (NIL);
          }
        }
      }
      else {
        if (((boolean)(match))) {
          return (consList(2, patternElement, match));
        }
        else {
          return (NIL);
        }
      }
    }
  }
}

Cons* matchPattern(Cons* pattern, Cons* datum, int datumLength) {
  if (pattern->length() < datumLength) {
    return (NIL);
  }
  else {
    { Cons* formList = NIL;
      Cons* patternCursor = pattern;
      Cons* datumCursor = datum;
      Object* patternElement = NULL;
      Object* datumElement = NULL;
      boolean optionalP = FALSE;
      Cons* matching = NIL;

      while (!((patternCursor == NIL) ||
          (datumCursor == NIL))) {
        patternElement = patternCursor->value;
        datumElement = datumCursor->value;
        optionalP = isaP(patternElement, SGT_DATE_TIME_PARSER_STELLA_CONS);
        matching = matchPatternElement((optionalP ? ((Symbol*)(((Cons*)(patternElement))->value)) : ((Symbol*)(patternElement))), datumElement);
        if (!(matching == NIL)) {
          { Object* formType = matching->value;

            if (!((formType == SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER) ||
                (formType == SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER))) {
              formList = cons(matching, formList);
            }
          }
        }
        else if (optionalP) {
          datumCursor = cons(datumElement, datumCursor);
        }
        else {
          return (NIL);
        }
        patternCursor = patternCursor->rest;
        datumCursor = datumCursor->rest;
      }
      if ((patternCursor == NIL) &&
          (datumCursor == NIL)) {
        return (formList->reverse());
      }
      else if (patternCursor == NIL) {
        return (NIL);
      }
      else if (datumCursor == NIL) {
        { Object* element = NULL;
          Cons* iter000 = patternCursor;

          while (!(iter000 == NIL)) {
            element = iter000->value;
            iter000 = iter000->rest;
            if (!isaP(element, SGT_DATE_TIME_PARSER_STELLA_CONS)) {
              return (NIL);
            }
          }
        }
        return (formList->reverse());
      }
      else {
        throw *newStellaException("Shouldn't get here!");
      }
    }
  }
}

void dealWithNoonMidn(Object* formValue, DecodedDateTime* parsedValues) {
  if (formValue == SYM_DATE_TIME_PARSER_STELLA_NOON) {
    parsedValues->dateTimeHour = 12;
  }
  else if (formValue == SYM_DATE_TIME_PARSER_STELLA_MIDN) {
    parsedValues->dateTimeHour = 0;
  }
  else {
    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "Unrecognized symbol: ~A" << "`" << formValue << "'";
      throw *newStellaException(stream000->theStringReader());
    }
  }
  parsedValues->dateTimeMinute = 0;
  parsedValues->dateTimeSecond = 0;
}

void dealWithAmPm(Object* formValue, DecodedDateTime* parsedValues) {
  { int hour = parsedValues->dateTimeHour;

    if (formValue == SYM_DATE_TIME_PARSER_STELLA_AM) {
      if (hour == 12) {
        parsedValues->dateTimeHour = 0;
      }
      else if (!((0 <= hour) &&
          (hour <= 12))) {
        if (oERROR_ON_MISMATCHo.get()) {
          { OutputStringStream* stream000 = newOutputStringStream();

            *(stream000->nativeStream) << "`" << hour << "'" << " is not an AM hour.";
            throw *newStellaException(stream000->theStringReader());
          }
        }
      }
    }
    else if (formValue == SYM_DATE_TIME_PARSER_STELLA_PM) {
      if ((0 <= hour) &&
          (hour <= 11)) {
        parsedValues->dateTimeHour = hour + 12;
      }
    }
    else {
      { OutputStringStream* stream001 = newOutputStringStream();

        *(stream001->nativeStream) << "`" << formValue << "'" << "isn't AM/PM - this shouldn't happen.";
        throw *newStellaException(stream001->theStringReader());
      }
    }
  }
}

void setTimeValues(Cons* stringForm, DecodedDateTime* parsedValues) {
  { Object* formType = NULL;
    Object* formValue = NULL;

    { Cons* formPart = NULL;
      Cons* iter000 = stringForm;

      while (!(iter000 == NIL)) {
        formPart = ((Cons*)(iter000->value));
        iter000 = iter000->rest;
        formType = formPart->value;
        formValue = formPart->rest->value;
        { GeneralizedSymbol* testValue000 = ((GeneralizedSymbol*)(formType));

          if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_SECONDP) {
            { Surrogate* testValue001 = safePrimaryType(formValue);

              if (subtypeOfIntegerP(testValue001)) {
                { Object* formValue000 = formValue;
                  IntegerWrapper* formValue = ((IntegerWrapper*)(formValue000));

                  parsedValues->dateTimeSecond = formValue->wrapperValue;
                }
              }
              else if (subtypeOfFloatP(testValue001)) {
                { Object* formValue001 = formValue;
                  FloatWrapper* formValue = ((FloatWrapper*)(formValue001));

                  parsedValues->dateTimeSecond = floor_(formValue->wrapperValue);
                  parsedValues->dateTimeMillis = round(1000.0 * (formValue->wrapperValue - floor_(formValue->wrapperValue)));
                }
              }
              else {
                { OutputStringStream* stream000 = newOutputStringStream();

                  *(stream000->nativeStream) << "`" << testValue001 << "'" << " is not a valid case option";
                  throw *newStellaException(stream000->theStringReader());
                }
              }
            }
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_MINUTE) {
            parsedValues->dateTimeMinute = ((IntegerWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_HOUR) {
            parsedValues->dateTimeHour = ((IntegerWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_DAY) {
            parsedValues->dateTimeDay = ((IntegerWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_MONTH) {
            parsedValues->dateTimeMonth = ((IntegerWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_YEAR) {
            parsedValues->dateTimeYear = ((IntegerWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_ZONE) {
            parsedValues->dateTimeZone = ((FloatWrapper*)(formValue))->wrapperValue;
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_WEEKDAY) {
            parsedValues->dateTimeDow = ((Keyword*)(formValue));
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_AM_PM) {
            dealWithAmPm(formValue, parsedValues);
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN) {
            dealWithNoonMidn(formValue, parsedValues);
          }
          else if (testValue000 == SYM_DATE_TIME_PARSER_STELLA_SPECIAL) {
            ((void  (*) (DecodedDateTime*))lookupFunction(((Symbol*)(formValue)))->functionCode)(parsedValues);
          }
          else {
            { OutputStringStream* stream001 = newOutputStringStream();

              *(stream001->nativeStream) << "Unrecognized symbol in form list: ~A." << "`" << formType << "'";
              throw *newStellaException(stream001->theStringReader());
            }
          }
        }
      }
    }
  }
}

DecodedDateTime* parseDateTime(char* dateTimeString, int start, int end, boolean errorOnMismatchP) {
  // Tries very hard to make sense out of the argument `date-time-string' and
  // returns a time structure if successful.  If not, it returns `null'.
  // If `error-on-mismatch?' is true, parse-date-time will signal an error instead of
  // returning `null'.  
  // Default values are 00:00:00 UTC on the current date
  { 
    BIND_STELLA_SPECIAL(oERROR_ON_MISMATCHo, boolean, errorOnMismatchP);
    { Cons* stringParts = decomposeDateTimeString(dateTimeString, start, end);
      int partsLength = stringParts->length();
      Cons* stringForm = NIL;

      { Object* pattern = NULL;
        Cons* iter000 = oDATE_TIME_PATTERNSo.get();

        while (!(iter000 == NIL)) {
          pattern = iter000->value;
          iter000 = iter000->rest;
          stringForm = matchPattern(((Cons*)(pattern)), stringParts, partsLength);
          if (!(stringForm == NIL)) {
            break;
          }
        }
      }
      if (stringForm == NIL) {
        if (oERROR_ON_MISMATCHo.get()) {
          { OutputStringStream* stream000 = newOutputStringStream();

            *(stream000->nativeStream) << "\"" << "`" << dateTimeString << "'" << "\" is not a recognized time/date format.";
            throw *newStellaException(stream000->theStringReader());
          }
        }
        else {
          return (NULL);
        }
      }
      else {
        { DecodedDateTime* parsedValues = makeCurrentDateTime()->decodeCalendarDate(getLocalTimeZone());

          parsedValues->dateTimeMillis = 0;
          parsedValues->dateTimeSecond = 0;
          parsedValues->dateTimeMinute = 0;
          parsedValues->dateTimeHour = 0;
          setTimeValues(stringForm, parsedValues);
          return (parsedValues);
        }
      }
    }
  }
}

CalendarDate* stringToCalendarDate(char* input) {
  // Returns a calendar date object representing the date and time
  // parsed from the `input' string.  If no valid parse is found,
  // `null' is returned.
  { DecodedDateTime* dO = parseDateTime(input, 0, NULL_INTEGER, FALSE);

    if (!((boolean)(dO))) {
      return (NULL);
    }
    else {
      return (dO->encodeCalendarDate());
    }
  }
}

void helpStartupDateTimeParser1() {
  {
    SYM_DATE_TIME_PARSER_STELLA_FOREACH = ((Symbol*)(internRigidSymbolWrtModule("FOREACH", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_ITEM = ((Symbol*)(internRigidSymbolWrtModule("ITEM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_IN = ((Symbol*)(internRigidSymbolWrtModule("IN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_CAST = ((Symbol*)(internRigidSymbolWrtModule("CAST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_CONS = ((Symbol*)(internRigidSymbolWrtModule("CONS", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_OF = ((Symbol*)(internRigidSymbolWrtModule("OF", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DO = ((Symbol*)(internRigidSymbolWrtModule("DO", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_INSERT_AT = ((Symbol*)(internRigidSymbolWrtModule("INSERT-AT", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_FIRST = ((Symbol*)(internRigidSymbolWrtModule("FIRST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_SECOND = ((Symbol*)(internRigidSymbolWrtModule("SECOND", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HASHLIST = ((Symbol*)(internRigidSymbolWrtModule("HASHLIST", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER = ((Symbol*)(internRigidSymbolWrtModule("HASHLIST-EXPANDER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_AM = ((Symbol*)(internRigidSymbolWrtModule("AM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_PM = ((Symbol*)(internRigidSymbolWrtModule("PM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_NOON = ((Symbol*)(internRigidSymbolWrtModule("NOON", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MIDN = ((Symbol*)(internRigidSymbolWrtModule("MIDN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MONTH = ((Symbol*)(internRigidSymbolWrtModule("MONTH", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_WEEKDAY = ((Symbol*)(internRigidSymbolWrtModule("WEEKDAY", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_AM_PM = ((Symbol*)(internRigidSymbolWrtModule("AM-PM", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN = ((Symbol*)(internRigidSymbolWrtModule("NOON-MIDN", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_ZONE = ((Symbol*)(internRigidSymbolWrtModule("ZONE", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_SPECIAL = ((Symbol*)(internRigidSymbolWrtModule("SPECIAL", NULL, 0)));
    KWD_DATE_TIME_PARSER_LETTER = ((Keyword*)(internRigidSymbolWrtModule("LETTER", NULL, 2)));
    KWD_DATE_TIME_PARSER_DIGIT = ((Keyword*)(internRigidSymbolWrtModule("DIGIT", NULL, 2)));
    SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER = ((Symbol*)(internRigidSymbolWrtModule("TIME-DIVIDER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER = ((Symbol*)(internRigidSymbolWrtModule("DATE-DIVIDER", NULL, 0)));
    SGT_DATE_TIME_PARSER_STELLA_CONS = ((Surrogate*)(internRigidSymbolWrtModule("CONS", NULL, 1)));
    SYM_DATE_TIME_PARSER_STELLA_SECONDP = ((Symbol*)(internRigidSymbolWrtModule("SECONDP", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_MINUTE = ((Symbol*)(internRigidSymbolWrtModule("MINUTE", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_HOUR = ((Symbol*)(internRigidSymbolWrtModule("HOUR", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_DAY = ((Symbol*)(internRigidSymbolWrtModule("DAY", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_YEAR = ((Symbol*)(internRigidSymbolWrtModule("YEAR", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-DATE-TIME-PARSER", NULL, 0)));
    SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void startupDateTimeParser() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      helpStartupDateTimeParser1();
    }
    if (currentStartupTimePhaseP(4)) {
      WHITESPACE_CHARS = getQuotedTree("((#\\  #\\Tab #\\Linefeed #\\, #\\' #\\`) \"/STELLA\")", "/STELLA");
      TIME_DIVIDERS = getQuotedTree("((#\\: #\\.) \"/STELLA\")", "/STELLA");
      DATE_DIVIDERS = getQuotedTree("((#\\\\ #\\/ #\\-) \"/STELLA\")", "/STELLA");
      oWEEKDAY_STRINGSo.set(newStringHashTable());
      oMONTH_STRINGSo.set(newStringToIntegerHashTable());
      oZONE_STRINGSo.set(newStringHashTable());
      oSPECIAL_STRINGSo.set(newStringHashTable());
      oSPECIAL_SYMBOLSo.set(getQuotedTree("((YESTERDAY TODAY TOMORROW NOW) \"/STELLA\")", "/STELLA"));
      { Cons* item = NULL;
        Cons* iter039 = ((Cons*)(getQuotedTree("(((\"monday\" :MONDAY) (\"mon\" :MONDAY) (\"tuesday\" :TUESDAY) (\"tues\" :TUESDAY) (\"tue\" :TUESDAY) (\"wednesday\" :WEDNESDAY) (\"wednes\" :WEDNESDAY) (\"wed\" :WEDNESDAY) (\"thursday\" :THURSDAY) (\"thurs\" :THURSDAY) (\"thu\" :THURSDAY) (\"friday\" :FRIDAY) (\"fri\" :FRIDAY) (\"saturday\" :SATURDAY) (\"sat\" :SATURDAY) (\"sunday\" :SUNDAY) (\"sun\" :SUNDAY)) \"/STELLA\")", "/STELLA")));

        while (!(iter039 == NIL)) {
          item = ((Cons*)(iter039->value));
          iter039 = iter039->rest;
          oWEEKDAY_STRINGSo.get()->insertAt(((StringWrapper*)(item->value))->wrapperValue, item->rest->value);
        }
      }
      { Cons* item = NULL;
        Cons* iter040 = ((Cons*)(getQuotedTree("(((\"january\" 1) (\"jan\" 1) (\"february\" 2) (\"feb\" 2) (\"march\" 3) (\"mar\" 3) (\"april\" 4) (\"apr\" 4) (\"may\" 5) (\"june\" 6) (\"jun\" 6) (\"july\" 7) (\"jul\" 7) (\"august\" 8) (\"aug\" 8) (\"september\" 9) (\"sept\" 9) (\"sep\" 9) (\"october\" 10) (\"oct\" 10) (\"november\" 11) (\"nov\" 11) (\"december\" 12) (\"dec\" 12)) \"/STELLA\")", "/STELLA")));

        while (!(iter040 == NIL)) {
          item = ((Cons*)(iter040->value));
          iter040 = iter040->rest;
          oMONTH_STRINGSo.get()->insertAt(((StringWrapper*)(item->value))->wrapperValue, ((IntegerWrapper*)(item->rest->value))->wrapperValue);
        }
      }
      { Cons* item = NULL;
        Cons* iter041 = ((Cons*)(getQuotedTree("(((\"gmt\" 0.0) (\"utc\" 0.0) (\"est\" -5.0) (\"edt\" -4.0) (\"cst\" -6.0) (\"cdt\" -5.0) (\"mst\" -7.0) (\"mdt\" -6.0) (\"pst\" -8.0) (\"pdt\" -7.0) (\"A\" 1.0) (\"B\" 2.0) (\"C\" 3.0) (\"C*\" 3.5) (\"D\" 4.0) (\"D*\" 4.5) (\"E\" 5.0) (\"E*\" 4.5) (\"F\" 6.0) (\"F*\" 6.5) (\"G\" 7.0) (\"H\" 7.0) (\"I\" 9.0) (\"I*\" 9.5) (\"K\" 10.0) (\"K*\" 10.5) (\"L\" 11.0) (\"L*\" 11.5) (\"M\" 12.0) (\"N\" -1.0) (\"O\" -2.0) (\"P\" -3.0) (\"P*\" -3.5) (\"Q\" -4.0) (\"R\" -5.0) (\"S\" -6.0) (\"T\" -7.0) (\"U\" -8.0) (\"U*\" -8.5) (\"V\" -9.0) (\"V*\" -9.5) (\"W\" -10.0) (\"X\" -11.0) (\"Y\" -12.0) (\"Z\" 0.0)) \"/STELLA\")", "/STELLA")));

        while (!(iter041 == NIL)) {
          item = ((Cons*)(iter041->value));
          iter041 = iter041->rest;
          oZONE_STRINGSo.get()->insertAt(((StringWrapper*)(item->value))->wrapperValue, item->rest->value);
        }
      }
      { Symbol* symbol = NULL;
        Cons* iter042 = oSPECIAL_SYMBOLSo.get();

        while (!(iter042 == NIL)) {
          symbol = ((Symbol*)(iter042->value));
          iter042 = iter042->rest;
          oSPECIAL_STRINGSo.get()->insertAt(stringDowncase(symbol->symbolName), symbol);
        }
      }
      oDATE_TIME_PATTERNSo.set(getQuotedTree("(((YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY (NOON-MIDN)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (NOON-MIDN (DATE-DIVIDER) (ZONE)) (HOUR (NOON-MIDN)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOON-MIDN (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOW) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH) (WEEKDAY MONTH DAY HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) SECONDP (AM-PM) (ZONE) YEAR) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) (MONTH)) (MONTH (DATE-DIVIDER) (YEAR)) ((WEEKDAY) MONTH DAY HOUR TIME-DIVIDER MINUTE (TIME-DIVIDER) (SECONDP) YEAR) (NOW (YESTERDAY)) ((YESTERDAY) NOW) (NOW (TODAY)) ((TODAY) NOW) (NOW (TOMORROW)) ((TOMORROW) NOW) (YESTERDAY (NOON-MIDN)) ((NOON-MIDN) YESTERDAY) (TODAY (NOON-MIDN)) ((NOON-MIDN) TODAY) (TOMORROW (NOON-MIDN)) ((NOON-MIDN) TOMORROW) (YESTERDAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TOMORROW HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TODAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE))) \"/STELLA\")", "/STELLA"));
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      defineFunctionObject("HASHLIST-EXPANDER", "(DEFUN (HASHLIST-EXPANDER OBJECT) ((LIST (CONS OF CONS)) (TABLE HASH-TABLE)))", ((cpp_function_code)(&hashlistExpander)), NULL);
      defineFunctionObject("YESTERDAY", "(DEFUN YESTERDAY ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&yesterday)), NULL);
      defineFunctionObject("TODAY", "(DEFUN TODAY ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&today)), NULL);
      defineFunctionObject("TOMORROW", "(DEFUN TOMORROW ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&tomorrow)), NULL);
      defineFunctionObject("NOW", "(DEFUN NOW ((PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&now)), NULL);
      defineFunctionObject("AM-PM", "(DEFUN (AM-PM SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&amPm)), NULL);
      defineFunctionObject("NOON-MIDN", "(DEFUN (NOON-MIDN SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&noonMidn)), NULL);
      defineFunctionObject("WEEKDAY", "(DEFUN (WEEKDAY KEYWORD) ((THING OBJECT)))", ((cpp_function_code)(&weekday)), NULL);
      defineFunctionObject("MONTH", "(DEFUN (MONTH INTEGER-WRAPPER) ((THING OBJECT)))", ((cpp_function_code)(&month)), NULL);
      defineFunctionObject("ZONE", "(DEFUN (ZONE FLOAT-WRAPPER) ((THING OBJECT)))", ((cpp_function_code)(&zone)), NULL);
      defineFunctionObject("SPECIALP", "(DEFUN (SPECIALP SYMBOL) ((STRING OBJECT)))", ((cpp_function_code)(&specialp)), NULL);
      defineFunctionObject("SECONDP", "(DEFUN (SECONDP BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&secondp)), NULL);
      defineFunctionObject("MINUTE", "(DEFUN (MINUTE BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&minute)), NULL);
      defineFunctionObject("HOUR", "(DEFUN (HOUR BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&hour)), NULL);
      defineFunctionObject("DAY", "(DEFUN (DAY BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&day)), NULL);
      defineFunctionObject("YEAR", "(DEFUN (YEAR BOOLEAN-WRAPPER) ((NUMBER OBJECT)))", ((cpp_function_code)(&year)), NULL);
      defineFunctionObject("TIME-DIVIDER", "(DEFUN (TIME-DIVIDER BOOLEAN-WRAPPER) ((THECHARACTER OBJECT)))", ((cpp_function_code)(&timeDivider)), NULL);
      defineFunctionObject("DATE-DIVIDER", "(DEFUN (DATE-DIVIDER BOOLEAN-WRAPPER) ((THECHARACTER OBJECT)))", ((cpp_function_code)(&dateDivider)), NULL);
      defineFunctionObject("MATCH-SUBSTRING", "(DEFUN (MATCH-SUBSTRING CONS) ((SUBSTRING STRING)))", ((cpp_function_code)(&matchSubstring)), NULL);
      defineFunctionObject("DIGIT-VALUE", "(DEFUN (DIGIT-VALUE INTEGER) ((CHAR CHARACTER)))", ((cpp_function_code)(&digitValue)), NULL);
      defineFunctionObject("DECOMPOSE-DATE-TIME-STRING", "(DEFUN (DECOMPOSE-DATE-TIME-STRING CONS) ((STRING STRING) (START INTEGER) (END INTEGER)))", ((cpp_function_code)(&decomposeDateTimeString)), NULL);
      defineFunctionObject("MATCH-PATTERN-ELEMENT", "(DEFUN (MATCH-PATTERN-ELEMENT CONS) ((PATTERN-ELEMENT SYMBOL) (DATUM-ELEMENT OBJECT)))", ((cpp_function_code)(&matchPatternElement)), NULL);
      defineFunctionObject("MATCH-PATTERN", "(DEFUN (MATCH-PATTERN CONS) ((PATTERN CONS) (DATUM CONS) (DATUM-LENGTH INTEGER)))", ((cpp_function_code)(&matchPattern)), NULL);
      defineFunctionObject("DEAL-WITH-NOON-MIDN", "(DEFUN DEAL-WITH-NOON-MIDN ((FORM-VALUE OBJECT) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&dealWithNoonMidn)), NULL);
      defineFunctionObject("DEAL-WITH-AM-PM", "(DEFUN DEAL-WITH-AM-PM ((FORM-VALUE OBJECT) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&dealWithAmPm)), NULL);
      defineFunctionObject("SET-TIME-VALUES", "(DEFUN SET-TIME-VALUES ((STRING-FORM (CONS OF CONS)) (PARSED-VALUES DECODED-DATE-TIME)))", ((cpp_function_code)(&setTimeValues)), NULL);
      defineFunctionObject("PARSE-DATE-TIME", "(DEFUN (PARSE-DATE-TIME DECODED-DATE-TIME) ((DATE-TIME-STRING STRING) (START INTEGER) (END INTEGER) (ERROR-ON-MISMATCH? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Tries very hard to make sense out of the argument `date-time-string' and\nreturns a time structure if successful.  If not, it returns `null'.\nIf `error-on-mismatch?' is true, parse-date-time will signal an error instead of\nreturning `null'.  \nDefault values are 00:00:00 UTC on the current date\")", ((cpp_function_code)(&parseDateTime)), NULL);
      defineFunctionObject("STRING-TO-CALENDAR-DATE", "(DEFUN (STRING-TO-CALENDAR-DATE CALENDAR-DATE) ((INPUT STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Returns a calendar date object representing the date and time\nparsed from the `input' string.  If no valid parse is found,\n`null' is returned.\")", ((cpp_function_code)(&stringToCalendarDate)), NULL);
      defineFunctionObject("STARTUP-DATE-TIME-PARSER", "(DEFUN STARTUP-DATE-TIME-PARSER () :PUBLIC? TRUE)", ((cpp_function_code)(&startupDateTimeParser)), NULL);
      { MethodSlot* function = lookupFunction(SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER);

        setDynamicSlotValue(function->dynamicSlots, SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Date-Time-Parser"), NULL_STRING_WRAPPER);
      }
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
    if (currentStartupTimePhaseP(9)) {
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT WHITESPACE-CHARS CONS (QUOTE (#\\  #\\Tab #\\Linefeed #\\, #\\' #\\`)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT TIME-DIVIDERS CONS (QUOTE (#\\: #\\.)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT DATE-DIVIDERS CONS (QUOTE (#\\\\ #\\/ #\\-)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *ERROR-ON-MISMATCH* BOOLEAN FALSE :PUBLIC? TRUE :DOCUMENTATION \"If TRUE, an error will be signalled if parse-date-time is unable\n   to determine the time/date format of the string.\")");
      storeMacro(SYM_DATE_TIME_PARSER_STELLA_HASHLIST, SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER, ((cpp_function_code)(&hashlistExpander)));
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *WEEKDAY-STRINGS* (STRING-HASH-TABLE OF STRING KEYWORD) (NEW (STRING-HASH-TABLE OF STRING KEYWORD)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *MONTH-STRINGS* STRING-TO-INTEGER-HASH-TABLE (NEW STRING-TO-INTEGER-HASH-TABLE))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *ZONE-STRINGS* (STRING-HASH-TABLE OF STRING FLOAT-WRAPPER) (NEW (STRING-HASH-TABLE OF STRING FLOAT-WRAPPER)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *SPECIAL-STRINGS* STRING-HASH-TABLE (NEW STRING-HASH-TABLE))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *SPECIAL-SYMBOLS* (CONS OF SYMBOL) (QUOTE (YESTERDAY TODAY TOMORROW NOW)))");
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *DATE-TIME-PATTERNS* CONS (QUOTE ((YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY (NOON-MIDN)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR (NOON-MIDN) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (NOON-MIDN) (DATE-DIVIDER) (ZONE)) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) ((NOON-MIDN) MONTH (DATE-DIVIDER) YEAR) ((NOON-MIDN) YEAR (DATE-DIVIDER) MONTH) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (NOON-MIDN (DATE-DIVIDER) (ZONE)) (HOUR (NOON-MIDN)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOON-MIDN (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY NOW) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) MONTH HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) (WEEKDAY) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH (DATE-DIVIDER) DAY) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) MONTH (DATE-DIVIDER) YEAR) (HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE) YEAR (DATE-DIVIDER) MONTH) (WEEKDAY MONTH DAY HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) SECONDP (AM-PM) (ZONE) YEAR) ((WEEKDAY) DAY (DATE-DIVIDER) MONTH (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) ((WEEKDAY) MONTH (DATE-DIVIDER) DAY (DATE-DIVIDER) YEAR HOUR (TIME-DIVIDER) MINUTE (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (YEAR (DATE-DIVIDER) (MONTH)) (MONTH (DATE-DIVIDER) (YEAR)) ((WEEKDAY) MONTH DAY HOUR TIME-DIVIDER MINUTE (TIME-DIVIDER) (SECONDP) YEAR) (NOW (YESTERDAY)) ((YESTERDAY) NOW) (NOW (TODAY)) ((TODAY) NOW) (NOW (TOMORROW)) ((TOMORROW) NOW) (YESTERDAY (NOON-MIDN)) ((NOON-MIDN) YESTERDAY) (TODAY (NOON-MIDN)) ((NOON-MIDN) TODAY) (TOMORROW (NOON-MIDN)) ((NOON-MIDN) TOMORROW) (YESTERDAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TOMORROW HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)) (TODAY HOUR (TIME-DIVIDER) (MINUTE) (TIME-DIVIDER) (SECONDP) (AM-PM) (DATE-DIVIDER) (ZONE)))))");
    }
  }
}

Symbol* SYM_DATE_TIME_PARSER_STELLA_FOREACH = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_ITEM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_IN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_CAST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_CONS = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_OF = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DO = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_INSERT_AT = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_FIRST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SECOND = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HASHLIST = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HASHLIST_EXPANDER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_AM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_PM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_NOON = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MIDN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MONTH = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_WEEKDAY = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_AM_PM = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_NOON_MIDN = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_ZONE = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SPECIAL = NULL;

Keyword* KWD_DATE_TIME_PARSER_LETTER = NULL;

Keyword* KWD_DATE_TIME_PARSER_DIGIT = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_TIME_DIVIDER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DATE_DIVIDER = NULL;

Surrogate* SGT_DATE_TIME_PARSER_STELLA_CONS = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_SECONDP = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_MINUTE = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_HOUR = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_DAY = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_YEAR = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_STARTUP_DATE_TIME_PARSER = NULL;

Symbol* SYM_DATE_TIME_PARSER_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
