// java-translate-file.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

void clt() {
  stringChangeModule("STELLA");
  setTranslatorOutputLanguage(KWD_JAVA_TRANSLATE_FILE_COMMON_LISP);
}

void jt() {
  stringChangeModule("STELLA");
  setTranslatorOutputLanguage(KWD_JAVA_TRANSLATE_FILE_JAVA);
}

void jptrans(Object* statement) {
  // Translate `statement' to C++ and print the result.
  { 
    BIND_STELLA_SPECIAL(oTRANSLATOROUTPUTLANGUAGEo, Keyword*, oTRANSLATOROUTPUTLANGUAGEo.get());
    BIND_STELLA_SPECIAL(oCURRENT_STREAMo, OutputStream*, STANDARD_OUTPUT);
    setTranslatorOutputLanguage(KWD_JAVA_TRANSLATE_FILE_JAVA);
    incrementallyTranslate(statement);
  }
}

char* stringJavaMakeCodeOutputFileName(char* barefile, boolean donttruncateP) {
  if (donttruncateP) {
    { 
      BIND_STELLA_SPECIAL(oDONTTRUNCATEFILENAMESpo, boolean, TRUE);
      return (makeFileNameFromRelativePath(wrapString(barefile), KWD_JAVA_TRANSLATE_FILE_JAVA));
    }
  }
  else {
    return (makeFileNameFromRelativePath(wrapString(barefile), KWD_JAVA_TRANSLATE_FILE_JAVA));
  }
}

char* Class::javaMakeCodeOutputFileName(boolean donttruncateP) {
  { Class* source = this;

    { StringWrapper* barefile = javaTranslateClassName(internSymbolInModule(source->classType->symbolName, ((Module*)(source->classType->homeContext)), TRUE));

      return (stringJavaMakeCodeOutputFileName(barefile->wrapperValue, donttruncateP));
    }
  }
}

char* javaMakeGlobalOutputFileName(Module* module, boolean donttruncateP) {
  return (stringJavaMakeCodeOutputFileName(javaYieldFlotsamClassName(module), donttruncateP));
}

void javaOutputFlotsamUnitsToFile(HashTable* globalsht, HashTable* functionht, Cons* codemodulelist) {
  { Cons* functions = NIL;
    Cons* globals = NIL;

    { Module* module = NULL;
      Cons* iter000 = codemodulelist;

      while (!(iter000 == NIL)) {
        module = ((Module*)(iter000->value));
        iter000 = iter000->rest;
        globals = ((Cons*)(globalsht->lookup(module)));
        if (!(((boolean)(globals)))) {
          globals = NIL;
        }
        functions = ((Cons*)(functionht->lookup(module)));
        if (!(((boolean)(functions)))) {
          functions = NIL;
        }
        javaOutputFlotsamUnitsToFileForModule(module, NULL, globals, functions);
      }
    }
  }
}

void javaOutputFlotsamUnitsToFileForModule(Module* module, TranslationUnit* classunit, Cons* globals, Cons* functions) {
  { char* classoutputfile = NULL;
    OutputFileStream* classoutputstream = NULL;
    Cons* translation = NIL;
    Class* clasS = NULL;
    Cons* classtranslation = NIL;

    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, module);
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
      classoutputfile = javaMakeGlobalOutputFileName(module, FALSE);
      classoutputstream = newOutputFileStream(classoutputfile);
      if (((boolean)(classunit))) {
        clasS = ((Class*)(classunit->theObject));
        cout << "Translating " << "`" << clasS << "'" << "..." << endl;
        classtranslation = javaTranslateDefineNativeClass(clasS)->rest;
      }
      { 
        BIND_STELLA_SPECIAL(oCURRENT_STREAMo, OutputStream*, classoutputstream);
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          if (((boolean)(classunit))) {
            cout << "    Writing " << "`" << classoutputfile << "'" << " ..." << endl;
          }
          else {
            cout << "Writing " << "`" << classoutputfile << "'" << "..." << endl;
          }
        }
        javaOutputFileHeader(classoutputstream, javaMakeGlobalOutputFileName(module, TRUE));
        if (((boolean)(classunit))) {
          javaOutputClassDeclaration(classtranslation);
        }
        else {
          *(oCURRENT_STREAMo.get()->nativeStream) << "public class " << javaYieldFlotsamClassName(module) << " ";
        }
        *(oCURRENT_STREAMo.get()->nativeStream) << "{" << endl;
        javaBumpIndent();
        if (((boolean)(classunit))) {
          javaOutputClassVariableDefinitions(((Cons*)(classtranslation->nth(6))));
        }
        { TranslationUnit* global = NULL;
          Cons* iter000 = globals;

          while (!(iter000 == NIL)) {
            global = ((TranslationUnit*)(iter000->value));
            iter000 = iter000->rest;
            translation = javaTranslateUnit(global);
            { Symbol* testValue000 = ((Symbol*)(translation->value));

              if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL) {
                javaOutputGlobalDefinition(translation->rest);
              }
              else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL) {
                javaOutputSpecialDefinition(translation->rest);
              }
              else {
                { OutputStringStream* stream000 = newOutputStringStream();

                  *(stream000->nativeStream) << "`" << testValue000 << "'" << " is not a valid case option";
                  throw *newStellaException(stream000->theStringReader());
                }
              }
            }
            global->translation = NULL;
            global->codeRegister = NULL;
          }
        }
        if (((boolean)(classunit))) {
          javaOutputClassConstructors(((Cons*)(classtranslation->nth(7))), javaYieldFlotsamClassName(module), ((boolean)(clasS)) &&
              exceptionClassP(clasS));
          { TranslationUnit* statement = NULL;
            Cons* iter001 = ((Cons*)(classtranslation->nth(8)));

            while (!(iter001 == NIL)) {
              statement = ((TranslationUnit*)(iter001->value));
              iter001 = iter001->rest;
              javaOutputStatement(javaTranslateUnit(statement));
              statement->translation = NULL;
              statement->codeRegister = NULL;
            }
          }
        }
        { TranslationUnit* function = NULL;
          Cons* iter002 = functions;

          while (!(iter002 == NIL)) {
            function = ((TranslationUnit*)(iter002->value));
            iter002 = iter002->rest;
            javaOutputMethod(javaTranslateUnit(function)->rest);
            function->translation = NULL;
            function->codeRegister = NULL;
          }
        }
        javaUnbumpIndent();
        *(oCURRENT_STREAMo.get()->nativeStream) << "}" << endl;
        classoutputstream->free();
      }
    }
  }
}

void javaOutputFileHeader(OutputStream* stream, char* filename) {
  *(stream->nativeStream) << "//  -*- Mode: Java -*-" << endl << "//" << endl;
  *(stream->nativeStream) << "// " << filename << endl << endl;
  { char* packagename = javaPackagePrefix(oMODULEo.get(), ".");

    outputCopyrightHeader(stream);
    if (!((packagename == NULL) ||
        stringEqlP(packagename, ""))) {
      packagename = stringSubsequence(packagename, 0, strlen(packagename) - 1);
      *(stream->nativeStream) << "package " << packagename << ";" << endl << endl;
    }
    if (!(oMODULEo.get() == oSTELLA_MODULEo)) {
      *(stream->nativeStream) << "import " << javaStellaPackage() << ".*;" << endl;
    }
    *(stream->nativeStream) << "import " << javaStellaPackage() << ".javalib.*;" << endl << endl;
  }
}

void javaOutputClassToFile(Class* clasS) {
  { 
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, clasS->homeModule());
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oCONTEXTo.get()->baseModule);
    { char* classoutputfile = clasS->javaMakeCodeOutputFileName(FALSE);
      OutputFileStream* classoutputstream = newOutputFileStream(classoutputfile);
      Cons* translation = NIL;

      { 
        BIND_STELLA_SPECIAL(oCURRENT_STREAMo, OutputStream*, classoutputstream);
        cout << "Translating " << "`" << clasS << "'" << "..." << endl;
        translation = javaTranslateDefineNativeClass(clasS);
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          cout << "    Writing " << "`" << classoutputfile << "'" << "..." << endl;
        }
        javaOutputFileHeader(classoutputstream, clasS->javaMakeCodeOutputFileName(TRUE));
        javaOutputClass(translation->rest, exceptionClassP(clasS));
        classoutputstream->free();
      }
    }
  }
}

void javaOutputClassUnitToFile(TranslationUnit* classunit) {
  javaOutputClassToFile(((Class*)(classunit->theObject)));
}

boolean javaFlotsamFunctionP(MethodSlot* method) {
  return ((!((BooleanWrapper*)(dynamicSlotValue(method->dynamicSlots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp, FALSE_WRAPPER)))->wrapperValue) &&
      ((!((BooleanWrapper*)(dynamicSlotValue(method->dynamicSlots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapperValue) &&
       ((!method->methodStartupFunctionP()) &&
        (zeroArgumentFunctionP(method) ||
         (javaMethodObjectDefinedOnNativeTypeP(method) ||
          javaFunctionInDifferentModuleP(method))))));
}

void javaOutputStartupUnitsToFile(StringHashTable* startupht, Cons* keylist) {
  { char* classoutputfile = NULL;
    OutputFileStream* classoutputstream = NULL;
    Cons* startupfunctions = NULL;

    { 
      BIND_STELLA_SPECIAL(oCURRENT_STREAMo, OutputStream*, NULL);
      { StringWrapper* classname = NULL;
        Cons* iter000 = keylist;

        while (!(iter000 == NIL)) {
          classname = ((StringWrapper*)(iter000->value));
          iter000 = iter000->rest;
          startupfunctions = ((Cons*)(startupht->lookup(classname->wrapperValue)));
          if (((boolean)(startupfunctions)) &&
              (!(!((boolean)(startupfunctions))))) {
            { 
              BIND_STELLA_SPECIAL(oMODULEo, Module*, ((TranslationUnit*)(startupfunctions->value))->homeModule());
              BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
              classoutputfile = stringJavaMakeCodeOutputFileName(classname->wrapperValue, FALSE);
              classoutputstream = newOutputFileStream(classoutputfile);
              oCURRENT_STREAMo.set(classoutputstream);
              if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
                cout << "Writing " << "`" << classoutputfile << "'" << "..." << endl;
              }
              javaOutputFileHeader(classoutputstream, stringJavaMakeCodeOutputFileName(classname->wrapperValue, TRUE));
              *(oCURRENT_STREAMo.get()->nativeStream) << "public class " << classname->wrapperValue << " {" << endl;
              javaBumpIndent();
              { TranslationUnit* function = NULL;
                Cons* iter001 = startupfunctions;

                while (!(iter001 == NIL)) {
                  function = ((TranslationUnit*)(iter001->value));
                  iter001 = iter001->rest;
                  javaOutputMethod(javaTranslateUnit(function)->rest);
                  function->translation = NULL;
                  function->codeRegister = NULL;
                }
              }
              javaUnbumpIndent();
              *(oCURRENT_STREAMo.get()->nativeStream) << "}" << endl;
              classoutputstream->free();
            }
          }
        }
      }
    }
  }
}

void javaPushIntoHashTable(HashTable* ht, Object* key, Object* object) {
  { Cons* value = ((Cons*)(ht->lookup(key)));

    if (((boolean)(value))) {
      ht->insertAt(key, cons(object, value));
    }
    else {
      ht->insertAt(key, cons(object, NIL));
    }
  }
}

void javaPushIntoStringHashTable(StringHashTable* ht, char* key, Object* object) {
  { Cons* value = ((Cons*)(ht->lookup(key)));

    if (((boolean)(value))) {
      ht->insertAt(key, cons(object, value));
    }
    else {
      ht->insertAt(key, cons(object, NIL));
    }
  }
}

boolean javaClassUnitDefinesFlotsamClassP(TranslationUnit* classunit) {
  { Class* clasS = ((Class*)(classunit->theObject));
    Module* module = classunit->homeModule();

    return (stringEqlP(javaTranslateClassNamestring(wrapString(internSymbolInModule(clasS->classType->symbolName, ((Module*)(clasS->classType->homeContext)), TRUE)->symbolName))->wrapperValue, javaYieldFlotsamClassName(module)));
  }
}

void javaOutputAllUnitsToFile() {
  { StringWrapper* startupclassname = NULL;
    StringHashTable* startupht = newStringHashTable();
    Cons* startupclasslist = NIL;
    Cons* methods = NIL;
    HashTable* flotsamfunctionht = newHashTable();
    HashTable* globalsht = newHashTable();
    List* codemoduleslist = list(0);
    Cons* verbatimstatements = NIL;
    Cons* classes = NIL;
    Module* codeoutputmodule = NULL;

    oJAVA_STELLA_PACKAGE_MAPPINGo->insertAt(wrapString("STELLAROOT"), wrapString(javaStellaPackage()));
    oTRANSLATIONUNITSo.set(oTRANSLATIONUNITSo.get()->reverse());
    { TranslationUnit* unit = NULL;
      Cons* iter000 = oTRANSLATIONUNITSo.get()->theConsList;

      while (!(iter000 == NIL)) {
        unit = ((TranslationUnit*)(iter000->value));
        iter000 = iter000->rest;
        { Symbol* testValue000 = unit->category;

          if ((testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD) ||
              (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO)) {
            if (javaFlotsamFunctionP(((MethodSlot*)(unit->theObject)))) {
              codeoutputmodule = unit->theObject->homeModule();
              javaPushIntoHashTable(flotsamfunctionht, codeoutputmodule, unit);
              codemoduleslist->insertNew(codeoutputmodule);
            }
            else if (((MethodSlot*)(unit->theObject))->methodStartupFunctionP()) {
              startupclassname = javaTranslateClassNamestring(wrapString(((StringWrapper*)(dynamicSlotValue(((MethodSlot*)(unit->theObject))->dynamicSlots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, NULL_STRING_WRAPPER)))->wrapperValue));
              javaPushIntoStringHashTable(startupht, startupclassname->wrapperValue, unit);
              if (!(startupclasslist->memberP(startupclassname))) {
                startupclasslist = cons(startupclassname, startupclasslist);
              }
            }
            else {
              methods = cons(unit, methods);
            }
          }
          else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD) {
            methods = cons(unit, methods);
          }
          else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE) {
          }
          else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM) {
            verbatimstatements = cons(unit, verbatimstatements);
          }
          else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS) {
            classes = cons(unit, classes);
          }
          else if (testValue000 == SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE) {
            codeoutputmodule = unit->theObject->homeModule();
            javaPushIntoHashTable(globalsht, codeoutputmodule, unit);
            codemoduleslist->insertNew(codeoutputmodule);
          }
          else {
            { OutputStringStream* stream000 = newOutputStringStream();

              *(stream000->nativeStream) << "`" << testValue000 << "'" << " is not a valid case option";
              throw *newStellaException(stream000->theStringReader());
            }
          }
        }
      }
    }
    { TranslationUnit* classunit = NULL;
      Cons* iter001 = classes;

      while (!(iter001 == NIL)) {
        classunit = ((TranslationUnit*)(iter001->value));
        iter001 = iter001->rest;
        if (javaClassUnitDefinesFlotsamClassP(classunit)) {
          { Module* module = classunit->theObject->homeModule();
            Cons* globals = NIL;
            Cons* functions = NIL;

            globals = ((Cons*)(globalsht->lookup(module)));
            if (!(((boolean)(globals)))) {
              globals = NIL;
            }
            functions = ((Cons*)(flotsamfunctionht->lookup(module)));
            if (!(((boolean)(functions)))) {
              functions = NIL;
            }
            javaOutputFlotsamUnitsToFileForModule(module, classunit, globals, functions);
            codemoduleslist->remove(module);
          }
        }
        else {
          javaOutputClassUnitToFile(classunit);
        }
      }
    }
    javaOutputFlotsamUnitsToFile(globalsht, flotsamfunctionht, codemoduleslist->theConsList);
    javaOutputStartupUnitsToFile(startupht, startupclasslist);
  }
}

void javaInitializeFileTranslation() {
  oJAVA_INDENT_CHARSo.set(0);
}

void javaTranslateFile(char* filename) {
  translateFile(filename, KWD_JAVA_TRANSLATE_FILE_JAVA, FALSE);
}

void javaTranslateFileAsPartOfSystem(char* filename) {
  translateFile(filename, KWD_JAVA_TRANSLATE_FILE_JAVA, TRUE);
}

void javaTranslateWalkedSystemUnits(KeyValueList* systemunits) {
  { 
    BIND_STELLA_SPECIAL(oTRANSLATIONUNITSo, List*, concatenateSystemUnits(systemunits));
    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Generating Java translations..." << endl;
    }
    javaOutputAllUnitsToFile();
    cleanUpTranslationUnitsSpecial();
  }
}

void javaTranslateSystem(char* systemname) {
  // Translate the system `systemName' to Java.
  translateSystem(systemname, KWD_JAVA_TRANSLATE_FILE_JAVA, 6, KWD_JAVA_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp, TRUE_WRAPPER);
}

void startupJavaTranslateFile() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      KWD_JAVA_TRANSLATE_FILE_COMMON_LISP = ((Keyword*)(internRigidSymbolWrtModule("COMMON-LISP", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_JAVA = ((Keyword*)(internRigidSymbolWrtModule("JAVA", NULL, 2)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL = ((Symbol*)(internRigidSymbolWrtModule("JAVA_GLOBAL", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL = ((Symbol*)(internRigidSymbolWrtModule("JAVA_SPECIAL", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp = ((Symbol*)(internRigidSymbolWrtModule("METHOD-CONSTRUCTOR?", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp = ((Symbol*)(internRigidSymbolWrtModule("METHOD-NATIVE?", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD = ((Symbol*)(internRigidSymbolWrtModule("METHOD", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO = ((Symbol*)(internRigidSymbolWrtModule("MACRO", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD = ((Symbol*)(internRigidSymbolWrtModule("PRINT-METHOD", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE = ((Symbol*)(internRigidSymbolWrtModule("TYPE", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM = ((Symbol*)(internRigidSymbolWrtModule("VERBATIM", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS = ((Symbol*)(internRigidSymbolWrtModule("CLASS", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(internRigidSymbolWrtModule("GLOBAL-VARIABLE", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
      KWD_JAVA_TRANSLATE_FILE_TWO_PASSp = ((Keyword*)(internRigidSymbolWrtModule("TWO-PASS?", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp = ((Keyword*)(internRigidSymbolWrtModule("FORCE-TRANSLATION?", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp = ((Keyword*)(internRigidSymbolWrtModule("PRODUCTION-SETTINGS?", NULL, 2)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-JAVA-TRANSLATE-FILE", NULL, 0)));
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      defineFunctionObject("CLT", "(DEFUN CLT ())", ((cpp_function_code)(&clt)), NULL);
      defineFunctionObject("JT", "(DEFUN JT ())", ((cpp_function_code)(&jt)), NULL);
      defineFunctionObject("JPTRANS", "(DEFUN JPTRANS ((STATEMENT OBJECT)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Translate `statement' to C++ and print the result.\")", ((cpp_function_code)(&jptrans)), NULL);
      defineMethodObject("(DEFMETHOD (JAVA-MAKE-CODE-OUTPUT-FILE-NAME FILE-NAME) ((BAREFILE FILE-NAME) (DONTTRUNCATE? BOOLEAN)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
      defineMethodObject("(DEFMETHOD (JAVA-MAKE-CODE-OUTPUT-FILE-NAME FILE-NAME) ((SOURCE CLASS) (DONTTRUNCATE? BOOLEAN)))", ((cpp_method_code)(&Class::javaMakeCodeOutputFileName)), ((cpp_method_code)(NULL)));
      defineFunctionObject("JAVA-MAKE-GLOBAL-OUTPUT-FILE-NAME", "(DEFUN (JAVA-MAKE-GLOBAL-OUTPUT-FILE-NAME FILE-NAME) ((MODULE MODULE) (DONTTRUNCATE? BOOLEAN)))", ((cpp_function_code)(&javaMakeGlobalOutputFileName)), NULL);
      defineFunctionObject("JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE ((GLOBALSHT HASH-TABLE) (FUNCTIONHT HASH-TABLE) (CODEMODULELIST (CONS OF MODULE))))", ((cpp_function_code)(&javaOutputFlotsamUnitsToFile)), NULL);
      defineFunctionObject("JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE-FOR-MODULE", "(DEFUN JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE-FOR-MODULE ((MODULE MODULE) (CLASSUNIT TRANSLATION-UNIT) (GLOBALS (CONS OF TRANSLATION-UNIT)) (FUNCTIONS (CONS OF TRANSLATION-UNIT))))", ((cpp_function_code)(&javaOutputFlotsamUnitsToFileForModule)), NULL);
      defineFunctionObject("JAVA-OUTPUT-FILE-HEADER", "(DEFUN JAVA-OUTPUT-FILE-HEADER ((STREAM OUTPUT-STREAM) (FILENAME FILE-NAME)))", ((cpp_function_code)(&javaOutputFileHeader)), NULL);
      defineFunctionObject("JAVA-OUTPUT-CLASS-TO-FILE", "(DEFUN JAVA-OUTPUT-CLASS-TO-FILE ((CLASS CLASS)))", ((cpp_function_code)(&javaOutputClassToFile)), NULL);
      defineFunctionObject("JAVA-OUTPUT-CLASS-UNIT-TO-FILE", "(DEFUN JAVA-OUTPUT-CLASS-UNIT-TO-FILE ((CLASSUNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&javaOutputClassUnitToFile)), NULL);
      defineFunctionObject("JAVA-FLOTSAM-FUNCTION?", "(DEFUN (JAVA-FLOTSAM-FUNCTION? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&javaFlotsamFunctionP)), NULL);
      defineFunctionObject("JAVA-OUTPUT-STARTUP-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-STARTUP-UNITS-TO-FILE ((STARTUPHT STRING-HASH-TABLE) (KEYLIST (CONS OF STRING-WRAPPER))))", ((cpp_function_code)(&javaOutputStartupUnitsToFile)), NULL);
      defineFunctionObject("JAVA-PUSH-INTO-HASH-TABLE", "(DEFUN JAVA-PUSH-INTO-HASH-TABLE ((HT HASH-TABLE) (KEY OBJECT) (OBJECT OBJECT)))", ((cpp_function_code)(&javaPushIntoHashTable)), NULL);
      defineFunctionObject("JAVA-PUSH-INTO-STRING-HASH-TABLE", "(DEFUN JAVA-PUSH-INTO-STRING-HASH-TABLE ((HT STRING-HASH-TABLE) (KEY STRING) (OBJECT OBJECT)))", ((cpp_function_code)(&javaPushIntoStringHashTable)), NULL);
      defineFunctionObject("JAVA-CLASS-UNIT-DEFINES-FLOTSAM-CLASS?", "(DEFUN (JAVA-CLASS-UNIT-DEFINES-FLOTSAM-CLASS? BOOLEAN) ((CLASSUNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&javaClassUnitDefinesFlotsamClassP)), NULL);
      defineFunctionObject("JAVA-OUTPUT-ALL-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-ALL-UNITS-TO-FILE ())", ((cpp_function_code)(&javaOutputAllUnitsToFile)), NULL);
      defineFunctionObject("JAVA-INITIALIZE-FILE-TRANSLATION", "(DEFUN JAVA-INITIALIZE-FILE-TRANSLATION ())", ((cpp_function_code)(&javaInitializeFileTranslation)), NULL);
      defineFunctionObject("JAVA-TRANSLATE-FILE", "(DEFUN JAVA-TRANSLATE-FILE ((FILENAME FILE-NAME)) :PUBLIC? TRUE)", ((cpp_function_code)(&javaTranslateFile)), NULL);
      defineFunctionObject("JAVA-TRANSLATE-FILE-AS-PART-OF-SYSTEM", "(DEFUN JAVA-TRANSLATE-FILE-AS-PART-OF-SYSTEM ((FILENAME FILE-NAME)))", ((cpp_function_code)(&javaTranslateFileAsPartOfSystem)), NULL);
      defineFunctionObject("JAVA-TRANSLATE-WALKED-SYSTEM-UNITS", "(DEFUN JAVA-TRANSLATE-WALKED-SYSTEM-UNITS ((SYSTEMUNITS SYSTEM-UNITS-ALIST)))", ((cpp_function_code)(&javaTranslateWalkedSystemUnits)), NULL);
      defineFunctionObject("JAVA-TRANSLATE-SYSTEM", "(DEFUN JAVA-TRANSLATE-SYSTEM ((SYSTEMNAME STRING)) :DOCUMENTATION \"Translate the system `systemName' to Java.\" :PUBLIC? TRUE)", ((cpp_function_code)(&javaTranslateSystem)), NULL);
      defineFunctionObject("STARTUP-JAVA-TRANSLATE-FILE", "(DEFUN STARTUP-JAVA-TRANSLATE-FILE () :PUBLIC? TRUE)", ((cpp_function_code)(&startupJavaTranslateFile)), NULL);
      { MethodSlot* function = lookupFunction(SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE);

        setDynamicSlotValue(function->dynamicSlots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Java-Translate-File"), NULL_STRING_WRAPPER);
      }
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
  }
}

Keyword* KWD_JAVA_TRANSLATE_FILE_COMMON_LISP = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_JAVA = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_TWO_PASSp = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE = NULL;

} // end of namespace stella
