// primal.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

boolean TRUE = 1;

boolean FALSE = 0;

double PI = 3.141592653589793;

int NULL_INTEGER = 1 << (sizeof (int) / sizeof (char) * 8 - 1);

short int NULL_SHORT_INTEGER = 1 << (sizeof (short int) / sizeof (char) * 8 - 1);

long int NULL_LONG_INTEGER = 1 << (sizeof (long int) / sizeof (char) * 8 - 1);

unsigned short int NULL_UNSIGNED_SHORT_INTEGER = (unsigned short int) -1;

unsigned long int NULL_UNSIGNED_LONG_INTEGER = (unsigned long int) -1;

double NULL_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

float NULL_SINGLE_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

double NULL_DOUBLE_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

char NULL_CHARACTER = '\0';

char NULL_BYTE = 255;

char NULL_OCTET = 255;

boolean eqlP(Object* x, Object* y) {
  // Return `true' if `x' and `y' are `eq?' or equivalent literals
  // such as strings that also might be wrapped in non-identical wrappers.  For
  // the case where `x' or `y' are plain literals such as strings or integers, the
  // STELLA translator substitutes the equality test appropriate for the particular
  // target language and does not actually call this function.  For cases where
  // `x' or `y' are known to be of type STANDARD-OBJECT, the STELLA translator
  // substitutes the faster `eq?' test inline.
  return (((!((boolean)(x))) ? (!((boolean)(y))) : ((x == y) ||
      x->objectEqlP(y))));
}

boolean equalP(Object* x, Object* y) {
  // Return `true' if `x' and `y' are `eql?' or considered equal
  // by a user-defined `object-equal?' method.  This implements a fully extensible
  // equality test similar to Java's `equals' method.
  return (((!((boolean)(x))) ? (!((boolean)(y))) : ((x == y) ||
      x->objectEqualP(y))));
}

boolean Object::objectEqualP(Object* y) {
  // Return `true' if `x' and `y' are `eq?'.
  { Object* x = this;

    return (x == y);
  }
}

boolean Wrapper::objectEqualP(Object* y) {
  // Return `true' if `x' and `y' are literal wrappers whose
  // literals are considered `eql?'.
  { Wrapper* x = this;

    return (x->objectEqlP(y));
  }
}

boolean zeroP(int x) {
  // Return `true' if `x' is 0.
  return ((!x));
}

boolean plusP(int x) {
  // Return `true' if `x' is greater than 0.
  return ((x > 0));
}

boolean evenP(int x) {
  // Return `true' if `x' is an even number.
  return (!(x % 2));
}

boolean oddP(int x) {
  // Return `true' if `x' is an odd number.
  return ((x % 2));
}

int mod(int x, int modulo) {
  // Return the result of `x' mod `modulo'.
  return ((x % modulo));
}

int random_(int n) {
  // Generate a random integer in the interval [0..n-1].
  // `n' must be <= 2^15.
  if (n > 32768) {
    throw *newStellaException("random: Can only generate random numbers between 0 and (2^15)-1.");
  }
  { int rnum = abs(rand()) % n;

    return (rnum);
  }
}

void seedRandomNumberGenerator() {
  // Seeds the random number generator with the current time.
  srand(time(NULL));;
}

double sqrt_(double n) {
  // Return the square root of `n'.
  return (sqrt(n));
}

double cos_(double n) {
  // Return the cosine of `n' radians.
  return (cos(n));
}

double sin_(double n) {
  // Return the sine of `n' radians.
  return (sin(n));
}

double tan_(double n) {
  // Return the tangent of `n' radians.
  return (tan(n));
}

double log_(double n) {
  // Return the natural logarithm (base e) of `n'.
  return (log(n));
}

int min_(int x, int y) {
  // Return the minimum of `x' and `y'.  If either is NULL, return the other.
  if (x == NULL_INTEGER) {
    return (y);
  }
  if (y == NULL_INTEGER) {
    return (x);
  }
  return (((x < y) ? x : y));
}

int max_(int x, int y) {
  // Return the maximum of `x' and `y'.  If either is NULL, return the other.
  if (x == NULL_INTEGER) {
    return (y);
  }
  if (y == NULL_INTEGER) {
    return (x);
  }
  return (((x > y) ? x : y));
}

int integerAbs(int x) {
  // Return the absolute value of `x'.
  return (((x < 0) ? (0 - x) : x));
}

double floatAbs(double x) {
  // Return the absolute value of `x'.
  return (((x < 0.0) ? (0.0 - x) : x));
}

int characterCode(char character) {
  // Return the 8-bit ASCII code of `character' as an integer.
  return ((int)(unsigned char) character);
}

char codeCharacter(int code) {
  // Return the character encoded by `code' (0 <= `code' <= 255).
  return ((char) code);
}

char characterDowncase(char c) {
  // Return the lowercase character for `c'.
  return ((isupper(c) ? c + 'a' - 'A' : c));
}

char characterUpcase(char c) {
  // Return the uppercase character for `c'.  If only the first character of
  // a sequence of characters is to be capitalized, character-capitalize should be used.
  return ((islower(c) ? c + 'A' - 'a' : c));
}

char characterCapitalize(char c) {
  // Return the capitalized character for `c'.  This is generally the same
  // as the uppercase character, except for obscure non-English characters in Java.  It should
  // be used if only the first character of a sequence of characters is to be capitalized.
  return ((islower(c) ? c + 'A' - 'a' : c));
}

DEFINE_STELLA_SPECIAL(oTRANSIENTOBJECTSpo, boolean , FALSE);

boolean stringEqlP(char* x, char* y) {
  // Return true if `x' and `y' are equal strings including case for both null.
  if (x == NULL) {
    return (y == NULL);
  }
  else {
    return ((y != NULL) &&
        !strcmp(x, y));
  }
}

boolean stringEqualP(char* x, char* y) {
  // Return true if `x' and `y' are equal strings ignoring case for both null.
  if (x == NULL) {
    return (y == NULL);
  }
  else {
    return ((y != NULL) &&
        !strcasecmp(x, y));
  }
}

int stringCompare(char* x, char* y, boolean caseSensitiveP) {
  // Compare `x' and `y' lexicographically, and return -1, 0, 
  // or 1, depending on whether `x' is less than, equal, or greater than `y'.
  // If `case-sensitive?' is `true', then case does matter for the comparison
  if (caseSensitiveP) {
    return (strcmp(x, y));
  }
  else {
    return (strcasecmp(x, y));
  }
}

boolean stringL(char* x, char* y) {
  // Return TRUE if `x' is lexicographically < `y', considering case.
  return (stringCompare(x, y, TRUE) < 0);
}

boolean stringLE(char* x, char* y) {
  // Return TRUE if `x' is lexicographically <= `y', considering case..
  return (stringCompare(x, y, TRUE) <= 0);
}

boolean stringGE(char* x, char* y) {
  // Return TRUE if `x' is lexicographically >= `y', considering case..
  return (stringCompare(x, y, TRUE) >= 0);
}

boolean stringG(char* x, char* y) {
  // Return TRUE if `x' is lexicographically > `y', considering case..
  return (stringCompare(x, y, TRUE) > 0);
}

boolean stringLessP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically < `y', ignoring case.
  return (stringCompare(x, y, FALSE) < 0);
}

boolean stringLessEqualP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically <= `y', ignoring case.
  return (stringCompare(x, y, FALSE) <= 0);
}

boolean stringGreaterEqualP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically >= `y', ignoring case.
  return (stringCompare(x, y, FALSE) >= 0);
}

boolean stringGreaterP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically > `y', ignoring case.
  return (stringCompare(x, y, FALSE) > 0);
}

char* makeMutableString(int size, char initchar) {
  // Return a new mutable string filled with `size' `initchar's.
  return (makeString(size, initchar));
}

char* makeRawMutableString(int size) {
  // Return a new uninitialized mutable string of `size'.
  { char* s = NULL;

    s = new (GC) char[size];
    return (s);
  }
}

char* stringConcatenate(char* string1, char* string2, int otherstrings, ...) {
  // Return a new string representing the concatenation
  // of `string1', `string2', and `otherStrings'.  The two mandatory parameters
  // allow us to optimize the common binary case by not relying on the somewhat
  // less efficient variable arguments mechanism.
  if (otherstrings == 0) {
    return (stringConcatenate(string1, string2));
  }
  { char* result = NULL;
    int length = strlen(string1) + strlen(string2);
    int index = 0;

    { char* string = NULL;
      va_list iter000;
      int iter000Count = otherstrings;

      va_start(iter000, otherstrings);
      while ((iter000Count--) > 0) {
        string = va_arg(iter000, char*);
        length = length + strlen(string);
      }
      va_end(iter000);
    }
    result = makeRawMutableString(length);
    { char ch = NULL_CHARACTER;
      char* vector000 = string1;
      int index000 = 0;
      int length000 = strlen(vector000);

      while (index000 < length000) {
        ch = vector000[index000];
        index000 = index000 + 1;
        result[index] = ch;
        index = index + 1;
      }
    }
    { char ch = NULL_CHARACTER;
      char* vector001 = string2;
      int index001 = 0;
      int length001 = strlen(vector001);

      while (index001 < length001) {
        ch = vector001[index001];
        index001 = index001 + 1;
        result[index] = ch;
        index = index + 1;
      }
    }
    { char* string = NULL;
      va_list iter001;
      int iter001Count = otherstrings;

      va_start(iter001, otherstrings);
      while ((iter001Count--) > 0) {
        string = va_arg(iter001, char*);
        { char ch = NULL_CHARACTER;
          char* vector002 = string;
          int index002 = 0;
          int length002 = strlen(vector002);

          while (index002 < length002) {
            ch = vector002[index002];
            index002 = index002 + 1;
            result[index] = ch;
            index = index + 1;
          }
        }
      }
      va_end(iter001);
    }
    return (result);
  }
}

char* replaceSubstrings(char* string, char* neW, char* old) {
  // Replace all occurrences of `old' in `string' with `new'.
  { int stringlength = strlen(string);
    int oldlength = strlen(old);
    int newlength = strlen(neW);
    int nofoccurrences = 0;
    int oldstart = 0;
    int cursor = 0;
    int resultcursor = 0;
    char* result = NULL;

    while ((oldstart = stringSearch(string, old, cursor)) != NULL_INTEGER) {
      nofoccurrences = nofoccurrences + 1;
      cursor = oldstart + oldlength;
    }
    if (nofoccurrences == 0) {
      return (string);
    }
    result = makeRawMutableString(stringlength + (nofoccurrences * (newlength - oldlength)));
    cursor = 0;
    while ((oldstart = stringSearch(string, old, cursor)) != NULL_INTEGER) {
      { int i = NULL_INTEGER;
        int iter000 = cursor;
        int upperBound000 = oldstart - 1;

        while (iter000 <= upperBound000) {
          i = iter000;
          iter000 = iter000 + 1;
          result[resultcursor] = (string[i]);
          resultcursor = resultcursor + 1;
        }
      }
      { char chaR = NULL_CHARACTER;
        char* vector000 = neW;
        int index000 = 0;
        int length000 = strlen(vector000);

        while (index000 < length000) {
          chaR = vector000[index000];
          index000 = index000 + 1;
          result[resultcursor] = chaR;
          resultcursor = resultcursor + 1;
        }
      }
      cursor = oldstart + oldlength;
    }
    { int i = NULL_INTEGER;
      int iter001 = cursor;
      int upperBound001 = stringlength - 1;

      while (iter001 <= upperBound001) {
        i = iter001;
        iter001 = iter001 + 1;
        result[resultcursor] = (string[i]);
        resultcursor = resultcursor + 1;
      }
    }
    return (result);
  }
}

int insertString(char* source, int start, int end, char* target, int targetIndex, Keyword* caseConversion) {
  // Inserts characters from `source' begining at `start' and
  // ending at `end' into `target' starting at `target-index'.  If `end' is `null',
  // then the entire length of the string is used. The copy of characters is affected
  // by the `case-conversion' keyword which should be one of
  //    :UPCASE :DOWNCASE :CAPITALIZE :PRESERVE.
  // 
  // The final value of target-index is returned.
  if (end == NULL_INTEGER) {
    end = strlen(source) - 1;
  }
  if (end < start) {
    return (targetIndex);
  }
  if (caseConversion == KWD_PRIMAL_UPCASE) {
    { int j = NULL_INTEGER;
      int iter000 = start;
      int upperBound000 = end;
      boolean unboundedP000 = upperBound000 == NULL_INTEGER;

      while (unboundedP000 ||
          (iter000 <= upperBound000)) {
        j = iter000;
        iter000 = iter000 + 1;
        target[targetIndex] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        targetIndex = targetIndex + 1;
      }
    }
  }
  else if (caseConversion == KWD_PRIMAL_DOWNCASE) {
    { int j = NULL_INTEGER;
      int iter001 = start;
      int upperBound001 = end;
      boolean unboundedP001 = upperBound001 == NULL_INTEGER;

      while (unboundedP001 ||
          (iter001 <= upperBound001)) {
        j = iter001;
        iter001 = iter001 + 1;
        target[targetIndex] = (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        targetIndex = targetIndex + 1;
      }
    }
  }
  else if (caseConversion == KWD_PRIMAL_CAPITALIZE) {
    target[targetIndex] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) (source[start]))]);
    targetIndex = targetIndex + 1;
    { int j = NULL_INTEGER;
      int iter002 = start + 1;
      int upperBound002 = end;
      boolean unboundedP002 = upperBound002 == NULL_INTEGER;

      while (unboundedP002 ||
          (iter002 <= upperBound002)) {
        j = iter002;
        iter002 = iter002 + 1;
        target[targetIndex] = (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        targetIndex = targetIndex + 1;
      }
    }
  }
  else if (caseConversion == KWD_PRIMAL_PRESERVE) {
    { int j = NULL_INTEGER;
      int iter003 = start;
      int upperBound003 = end;
      boolean unboundedP003 = upperBound003 == NULL_INTEGER;

      while (unboundedP003 ||
          (iter003 <= upperBound003)) {
        j = iter003;
        iter003 = iter003 + 1;
        target[targetIndex] = (source[j]);
        targetIndex = targetIndex + 1;
      }
    }
  }
  else {
    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "`" << caseConversion << "'" << " is not a valid case option";
      throw *newStellaException(stream000->theStringReader());
    }
  }
  return (targetIndex);
}

char* characterToString(char c) {
  return (makeString(1, c));
}

char* formatWithPadding(char* input, int length, char padchar, Keyword* align, boolean truncateP) {
  // Formats `input' to be (at least) `length' long, using `padchar' to
  // fill if necessary.  `align' must be one of :LEFT, :RIGHT, :CENTER and will control
  // how `input' will be justified in the resulting string.  If `truncate?' is `true', then
  // then an overlength string will be truncated, using the opposite of `align' to pick
  // the truncation direction.
  { int len = strlen(input);

    if (len == length) {
      return (input);
    }
    else if (len > length) {
      if (truncateP) {
        if (align == KWD_PRIMAL_LEFT) {
          return (stringSubsequence(input, len - length, len));
        }
        else if (align == KWD_PRIMAL_RIGHT) {
          return (stringSubsequence(input, 0, len - length));
        }
        else if (align == KWD_PRIMAL_CENTER) {
          { int left = ((int)((len - length) / 2.0));

            return (stringSubsequence(input, left, left + length));
          }
        }
        else {
          { OutputStringStream* stream000 = newOutputStringStream();

            *(stream000->nativeStream) << "`" << align << "'" << " is not a valid case option";
            throw *newStellaException(stream000->theStringReader());
          }
        }
      }
      else {
        return (input);
      }
    }
    else {
      if (align == KWD_PRIMAL_LEFT) {
        return (stringConcatenate(input, makeString(length - len, padchar), 0));
      }
      else if (align == KWD_PRIMAL_RIGHT) {
        return (stringConcatenate(makeString(length - len, padchar), input, 0));
      }
      else if (align == KWD_PRIMAL_CENTER) {
        { int left = ((int)((length - len) / 2.0));

          return (stringConcatenate(makeString(left, padchar), input, 1, makeString((length - len) - left, padchar)));
        }
      }
      else {
        { OutputStringStream* stream001 = newOutputStringStream();

          *(stream001->nativeStream) << "`" << align << "'" << " is not a valid case option";
          throw *newStellaException(stream001->theStringReader());
        }
      }
    }
  }
}

char stringNth(char* self, int position) {
  // Return the character in `self' at `position'.
  return (self[position]);
}

char mutableStringNth(char* self, int position) {
  // Return the character in `self' at `position'.
  return (self[position]);
}

char mutableStringNthSetter(char* self, char ch, int position) {
  // Set the character in `self' at `position' to `ch'.
  return (self[position] = ch);
}

int stringLength(char* self) {
  // Return the length of the string `self'.
  return (strlen(self));
}

int mutableStringLength(char* self) {
  // Return the length of the string `self'.
  return (strlen(self));
}

Object* unstringify(char* string) {
  return (readSExpressionFromString(string));
}

# include "stella/cpp-lib/cpp-hashtable.hh"
void AbstractHashTable::freeHashTableValues() {
  // Call free on each value in the hash table `self'.
  { AbstractHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      cout << "WARNING: freeHashTableValues not yet implemented" << endl;;
    }
  }
}

void AbstractHashTable::initializeHashTable() {
  // Insert a newly-created native hash table into `self'.
  { AbstractHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->theStellaHashTable = makeStellaHashTable(50);
      return;
    }
    self->theHashTable = (cpp_hash_table*)(new Native_EQL_Hash_Table<Object*, Object*>);
  }
}

Object* HashTable::lookup(Object* key) {
  { HashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stellaHashTableLookup(self->theStellaHashTable, key));
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      return (((Native_EQL_Hash_Table<Object*, Object*>*)nativeTable)->get(key));
    }
  }
}

void HashTable::insertAt(Object* key, Object* value) {
  { HashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableInsertAt(self->theStellaHashTable, key, value);
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      (*((Native_EQL_Hash_Table<Object*, Object*>*)nativeTable))[key] = value;;
    }
  }
}

void HashTable::removeAt(Object* key) {
  { HashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableRemoveAt(self->theStellaHashTable, key);
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      ((Native_EQL_Hash_Table<Object*, Object*>*)nativeTable)->erase(key);;
    }
  }
}

void IntegerHashTable::initializeHashTable() {
  // Insert a newly-created native hash table into `self'.
  { IntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->theStellaHashTable = makeStellaHashTable(50);
      return;
    }
    self->theHashTable = ((cpp_hash_table*)(new Native_EQL_Hash_Table<int, Object*>));
  }
}

Object* IntegerHashTable::lookup(int key) {
  { IntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stellaHashTableLookup(self->theStellaHashTable, wrapInteger(key)));
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      return (((Native_EQL_Hash_Table<int, Object*>*)nativeTable)->get(key));
    }
  }
}

void IntegerHashTable::insertAt(int key, Object* value) {
  { IntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableInsertAt(self->theStellaHashTable, wrapInteger(key), value);
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      (*((Native_EQL_Hash_Table<int, Object*>*)nativeTable))[key] = value;;
    }
  }
}

void IntegerHashTable::removeAt(int key) {
  { IntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableRemoveAt(self->theStellaHashTable, wrapInteger(key));
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      ((Native_EQL_Hash_Table<int, Object*>*)nativeTable)->erase(key);;
    }
  }
}

void FloatHashTable::initializeHashTable() {
  // Insert a newly-created native hash table into `self'.
  { FloatHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->theStellaHashTable = makeStellaHashTable(50);
      return;
    }
    self->theHashTable = (cpp_hash_table*)(new Native_EQL_Hash_Table<double, Object*>);
  }
}

Object* FloatHashTable::lookup(double key) {
  { FloatHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stellaHashTableLookup(self->theStellaHashTable, wrapFloat(key)));
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      return (((Native_EQL_Hash_Table<double, Object*>*)nativeTable)->get(key));
    }
  }
}

void FloatHashTable::insertAt(double key, Object* value) {
  { FloatHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableInsertAt(self->theStellaHashTable, wrapFloat(key), value);
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      (*((Native_EQL_Hash_Table<double, Object*>*)nativeTable))[key] = value;;
    }
  }
}

void FloatHashTable::removeAt(double key) {
  { FloatHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableRemoveAt(self->theStellaHashTable, wrapFloat(key));
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      ((Native_EQL_Hash_Table<double, Object*>*)nativeTable)->erase(key);;
    }
  }
}

void StringHashTable::initializeHashTable() {
  // Insert a newly-created native hash table into `self'.
  { StringHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->theStellaHashTable = makeStellaHashTable(50);
      return;
    }
    self->theHashTable = (cpp_hash_table*)(new Native_EQL_Hash_Table<char*, Object*>);
  }
}

Object* StringHashTable::lookup(char* key) {
  { StringHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stellaStringHashTableLookup(self->theStellaHashTable, key));
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      return (((Native_EQL_Hash_Table<char*, Object*>*)nativeTable)->get(key));
    }
  }
}

void StringHashTable::insertAt(char* key, Object* value) {
  { StringHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableInsertAt(self->theStellaHashTable, wrapString(key), value);
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      (*((Native_EQL_Hash_Table<char*, Object*>*)nativeTable))[key] = value;;
    }
  }
}

void StringHashTable::removeAt(char* key) {
  { StringHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableRemoveAt(self->theStellaHashTable, wrapString(key));
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      ((Native_EQL_Hash_Table<char*, Object*>*)nativeTable)->erase(key);;
    }
  }
}

void StringToIntegerHashTable::initializeHashTable() {
  // Insert a newly-created native hash table into `self'.
  { StringToIntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->theStellaHashTable = makeStellaHashTable(50);
      return;
    }
    self->theHashTable = (cpp_hash_table*)(new Native_EQL_Hash_Table<char*, int>(NULL_INTEGER));
  }
}

int StringToIntegerHashTable::lookup(char* key) {
  { StringToIntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (unwrapInteger(((IntegerWrapper*)(stellaStringHashTableLookup(self->theStellaHashTable, key)))));
    }
    { cpp_hash_table* nativeTable = self->theHashTable;
      int result = NULL_INTEGER;

      result = ((Native_EQL_Hash_Table<char*, int>*)nativeTable)->get(key);
      return (result);
    }
  }
}

void StringToIntegerHashTable::insertAt(char* key, int value) {
  { StringToIntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableInsertAt(self->theStellaHashTable, wrapString(key), wrapInteger(value));
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      (*((Native_EQL_Hash_Table<char*, int>*)nativeTable))[key] = value;;
    }
  }
}

void StringToIntegerHashTable::removeAt(char* key) {
  { StringToIntegerHashTable* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stellaHashTableRemoveAt(self->theStellaHashTable, wrapString(key));
      return;
    }
    { cpp_hash_table* nativeTable = self->theHashTable;

      ((Native_EQL_Hash_Table<char*, int>*)nativeTable)->erase(key);;
    }
  }
}

// A table of 256 26-bit random numbers that can be used to
// hash sequences of bytes.  Each bit-column in the table has an approximately
// even number of 0's and 1's.
int* oHASH_BYTE_RANDOM_TABLEo = NULL;

int objectHashCode(Object* self) {
  // Return a hash code for `self'.  Two objects that are `eq?'
  // are guaranteed to generate the same hash code.  Two objects that are not `eq?'
  // do not necessarily generate different hash codes.  Similar to `hash-code' but
  // always hashes on the address of `self' even if it is a wrapper.
  return ((size_t)self);
}

int Object::hashCode() {
  // Return a hash code for `self'.  Two objects that are `eql?'
  // are guaranteed to generate the same hash code.  Two objects that are not `eql?'
  // do not necessarily generate different hash codes.
  { Object* self = this;

    return ((size_t)self);
  }
}

int StandardObject::hashCode() {
  { StandardObject* self = this;

    return ((size_t)self);
  }
}

int Wrapper::hashCode() {
  { Wrapper* self = this;

    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "WRAPPER.hash-code: Don't know how to hash on " << "`" << self << "'";
      throw *newStellaException(stream000->theStringReader());
    }
  }
}

int StringWrapper::hashCode() {
  { StringWrapper* self = this;

    return (hashString(self->wrapperValue, 0));
  }
}

int IntegerWrapper::hashCode() {
  { IntegerWrapper* self = this;

    return (self->wrapperValue);
  }
}

int FloatWrapper::hashCode() {
  { FloatWrapper* self = this;

    return ((size_t)(self->wrapperValue));
  }
}

int CharacterWrapper::hashCode() {
  { CharacterWrapper* self = this;

    return (((oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) (self->wrapperValue)]) ^ 33485726));
  }
}

int BooleanWrapper::hashCode() {
  { BooleanWrapper* self = this;

    return ((self->wrapperValue ? 27333705 : 31891526));
  }
}

int stringHashCode(char* self) {
  return (hashString(self, 0));
}

int integerHashCode(int self) {
  return (self);
}

int floatHashCode(double self) {
  return ((size_t)self);
}

int characterHashCode(char self) {
  return (((oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) self]) ^ 33485726));
}

int hashString(char* string, int seedcode) {
  // Generate a hash-code for `string' and return it.
  // Two strings that are equal but not eq will generate the same code.
  // The hash-code is based on `seedCode' which usually will be 0.  However,
  // `seedCode' can also be used to supply the result of a previous hash
  // operation to achieve hashing on sequences of strings without actually
  // having to concatenate them.
  { int code = seedcode;
    int cursor = strlen(string);

    if (cursor == 0) {
      return (4303803);
    }
    else {
      cursor = cursor - 1;
    }
    for (;;) {
      code = (code ^ (oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) (string[cursor])]));
      if (cursor == 0) {
        break;
      }
      else {
        cursor = cursor - 1;
      }
      if ((code & 1) == 0) {
        code = (code >> 1);
      }
      else {
        code = (((code >> 1)) | 67108864);
      }
    }
    return (code);
  }
}

// List of prime numbers approximately growing by a factor of 2
// that are suitable to be used as hash table sizes.
List* oHASH_TABLE_SIZE_PRIMESo = NULL;

int pickHashTableSizePrime(int minsize) {
  // Return a hash table prime of at least `minSize'.
  { IntegerWrapper* prime = NULL;
    Cons* iter000 = oHASH_TABLE_SIZE_PRIMESo->theConsList;

    while (!(iter000 == NIL)) {
      prime = ((IntegerWrapper*)(iter000->value));
      iter000 = iter000->rest;
      if (prime->wrapperValue >= minsize) {
        return (prime->wrapperValue);
      }
    }
  }
  throw *newStellaException("pick-hash-table-size-prime: minimum size is too large");
}

void Vector::initializeVector() {
  { Vector* self = this;

    { int size = self->arraySize;

      self->theArray = new (GC) Object*[size];
      { Object** array = self->theArray;

        { int i = NULL_INTEGER;
          int iter000 = 0;
          int upperBound000 = size - 1;

          while (iter000 <= upperBound000) {
            i = iter000;
            iter000 = iter000 + 1;
            array[i] = NULL;
          }
        }
      }
    }
  }
}

void resizeVector(Vector* self, int size) {
  // Change the size of `self' to `size'.  If `size' is smaller
  // than the current size of `self' the vector will be truncated.  Otherwise, 
  // the internal array of `self' will be grown to `size' and unused elements
  // will be initialized to NULL.
  { int old_size = self->arraySize;
    int i = old_size;
    Object** old_array = self->theArray;
    Object** new_array = old_array;

    if (size == old_size) {
      return;
    }
    new_array = new (GC) Object*[size];
    if (size < old_size) {
      old_size = size;
      i = old_size;
    }
    while ((i = i - 1) >= 0) {
      new_array[i] = (old_array[i]);
    }
    { int i = NULL_INTEGER;
      int iter000 = old_size;
      int upperBound000 = size - 1;

      while (iter000 <= upperBound000) {
        i = iter000;
        iter000 = iter000 + 1;
        new_array[i] = NULL;
      }
    }
    self->theArray = new_array;
    self->arraySize = size;
  }
}

Object* nativeVectorNth(Object** self, int position) {
  // Return the element in `self' at `position'.
  return (self[position]);
}

Object* nativeVectorNthSetter(Object** self, Object* value, int position) {
  // Set the element in `self' at `position' to `value'.
  return (self[position] = value);
}

void unmake(Object* self) {
}

void copyFile(char* fromfile, char* tofile) {
  // Copy file `fromFile' to file `toFile', clobbering
  // any data already in `toFile'.
  if (!probeFileP(fromfile)) {
    { OutputStringStream* stream000 = newOutputStringStream();

      *(stream000->nativeStream) << "Can't copy non-existent file: " << "`" << fromfile << "'" << endl;
      throw *newStellaException(stream000->theStringReader());
    }
  }
  { InputFileStream* from = newInputFileStream(fromfile);
    OutputFileStream* to = newOutputFileStream(tofile);

    { char* line = NULL;
      LineIterator* iter000 = lines(from);

      while (iter000->nextP()) {
        line = iter000->value;
        *(to->nativeStream) << line << endl;
      }
    }
    to->free();
  }
}

int lognot(int arg) {
  return ((~ arg));
}

int logand(int arg1, int arg2) {
  return ((arg1 & arg2));
}

int logor(int arg1, int arg2) {
  return ((arg1 | arg2));
}

int logxor(int arg1, int arg2) {
  return ((arg1 ^ arg2));
}

int shiftLeft(int arg, int count) {
  return ((arg << count));
}

int shiftRight(int arg, int count) {
  return ((arg >> count));
}

Object* stellify(Object* self) {
  // Convert a Lisp object into a STELLA object.
  { Object* result = self;

    return (result);
  }
}

boolean stellaObjectP(Object* self) {
  // Return TRUE if `self' is a member of the STELLA class `OBJECT'.
  { boolean resultP = TRUE;

    return (resultP);
  }
}

boolean runningAsLispP() {
  // Return TRUE if the executable code is a Common Lisp application.
  { boolean resultP = FALSE;

    return (resultP);
  }
}

void sleep_(double seconds) {
  // The program will sleep for the indicated number of seconds.
  // Fractional values are allowed, but the results are implementation dependent:
  // Common Lisp uses the fractions natively, Java with a resolution of 0.001,
  // and C++ can only use integral values.
  sleep((unsigned int)seconds);
}

char* exceptionMessage(exception* e) {
  // Accesses the error message of the exception `e'.
  return ((char*)e->what());
}

void printExceptionContext(exception* e, OutputStream* stream) {
  // Prints a system dependent information about the context of the specified
  // exception.  For example, in Java it prints a stack trace.  In Lisp, it is vendor dependent.
  { ostream* s = stream->nativeStream;

    *(stream->nativeStream) << "No exception context available" << endl;
  }
}

char* makeProcessLock() {
  return (NULL);
}

void helpStartupPrimal1() {
  {
    SYM_PRIMAL_STELLA_FLOOR = ((Symbol*)(internRigidSymbolWrtModule("FLOOR", NULL, 0)));
    KWD_PRIMAL_CPP = ((Keyword*)(internRigidSymbolWrtModule("CPP", NULL, 2)));
    KWD_PRIMAL_FUNCTION = ((Keyword*)(internRigidSymbolWrtModule("FUNCTION", NULL, 2)));
    SYM_PRIMAL_STELLA_RANDOM = ((Symbol*)(internRigidSymbolWrtModule("RANDOM", NULL, 0)));
    SYM_PRIMAL_STELLA_SQRT = ((Symbol*)(internRigidSymbolWrtModule("SQRT", NULL, 0)));
    SYM_PRIMAL_STELLA_COS = ((Symbol*)(internRigidSymbolWrtModule("COS", NULL, 0)));
    SYM_PRIMAL_STELLA_SIN = ((Symbol*)(internRigidSymbolWrtModule("SIN", NULL, 0)));
    SYM_PRIMAL_STELLA_TAN = ((Symbol*)(internRigidSymbolWrtModule("TAN", NULL, 0)));
    SYM_PRIMAL_STELLA_LOG = ((Symbol*)(internRigidSymbolWrtModule("LOG", NULL, 0)));
    SYM_PRIMAL_STELLA_MIN = ((Symbol*)(internRigidSymbolWrtModule("MIN", NULL, 0)));
    SYM_PRIMAL_STELLA_MAX = ((Symbol*)(internRigidSymbolWrtModule("MAX", NULL, 0)));
    KWD_PRIMAL_UPCASE = ((Keyword*)(internRigidSymbolWrtModule("UPCASE", NULL, 2)));
    KWD_PRIMAL_DOWNCASE = ((Keyword*)(internRigidSymbolWrtModule("DOWNCASE", NULL, 2)));
    KWD_PRIMAL_CAPITALIZE = ((Keyword*)(internRigidSymbolWrtModule("CAPITALIZE", NULL, 2)));
    KWD_PRIMAL_PRESERVE = ((Keyword*)(internRigidSymbolWrtModule("PRESERVE", NULL, 2)));
    KWD_PRIMAL_LEFT = ((Keyword*)(internRigidSymbolWrtModule("LEFT", NULL, 2)));
    KWD_PRIMAL_RIGHT = ((Keyword*)(internRigidSymbolWrtModule("RIGHT", NULL, 2)));
    KWD_PRIMAL_CENTER = ((Keyword*)(internRigidSymbolWrtModule("CENTER", NULL, 2)));
    SYM_PRIMAL_STELLA_HASH_CODE = ((Symbol*)(internRigidSymbolWrtModule("HASH-CODE", NULL, 0)));
    KWD_PRIMAL_JAVA = ((Keyword*)(internRigidSymbolWrtModule("JAVA", NULL, 2)));
    SYM_PRIMAL_STELLA_SLEEP = ((Symbol*)(internRigidSymbolWrtModule("SLEEP", NULL, 0)));
    SYM_PRIMAL_STELLA_STARTUP_PRIMAL = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-PRIMAL", NULL, 0)));
    SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void helpStartupPrimal2() {
  {
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST UNKNOWN)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST SECOND-CLASS-OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST NATIVE-VECTOR)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST MUTABLE-STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST CHARACTER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST CODE)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST INTEGER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NULL? BOOLEAN) ((TEST FLOAT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST UNKNOWN)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST SECOND-CLASS-OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST NATIVE-VECTOR)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST MUTABLE-STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST CHARACTER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST CODE)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST INTEGER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST FLOAT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("EQ?", "(DEFUN (EQ? BOOLEAN) ((X UNKNOWN) (Y UNKNOWN)) :DOCUMENTATION \"Return `true' if `x' and `y' are literally the same object (or\nsimple number).  Analogue to the Common Lisp EQL and C++ and Java's ==.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("=", "(DEFUN (= BOOLEAN) ((X NUMBER) (Y NUMBER)) :DOCUMENTATION \"Return `true' if `x' and `y' are numbers of exactly the same magnitude.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("EQL?", "(DEFUN (EQL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eq?' or equivalent literals\nsuch as strings that also might be wrapped in non-identical wrappers.  For\nthe case where `x' or `y' are plain literals such as strings or integers, the\nSTELLA translator substitutes the equality test appropriate for the particular\ntarget language and does not actually call this function.  For cases where\n`x' or `y' are known to be of type STANDARD-OBJECT, the STELLA translator\nsubstitutes the faster `eq?' test inline.\" :PUBLIC? TRUE)", ((cpp_function_code)(&eqlP)), NULL);
    defineFunctionObject("EQUAL?", "(DEFUN (EQUAL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eql?' or considered equal\nby a user-defined `object-equal?' method.  This implements a fully extensible\nequality test similar to Java's `equals' method.\" :PUBLIC? TRUE)", ((cpp_function_code)(&equalP)), NULL);
    defineMethodObject("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eq?'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::objectEqualP)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X WRAPPER) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are literal wrappers whose\nliterals are considered `eql?'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Wrapper::objectEqualP)), ((cpp_method_code)(NULL)));
    defineFunctionObject(">", "(DEFUN (> BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject(">=", "(DEFUN (>= BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("<", "(DEFUN (< BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("<=", "(DEFUN (<= BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("+", "(DEFUN (+ NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("-", "(DEFUN (- NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("*", "(DEFUN (* NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("/", "(DEFUN (/ NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("ZERO?", "(DEFUN (ZERO? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:= X 0) :CPP \"(!x)\" :JAVA \"(x == 0)\")))", ((cpp_function_code)(&zeroP)), NULL);
    defineFunctionObject("PLUS?", "(DEFUN (PLUS? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is greater than 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:> X 0) :CPP \"(x > 0)\" :JAVA \"(x > 0)\")))", ((cpp_function_code)(&plusP)), NULL);
    defineFunctionObject("EVEN?", "(DEFUN (EVEN? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is an even number.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:NOT (CL:LOGBITP 0 X)) :CPP \"!(x % 2)\" :JAVA \"((x % 2) == 0)\")))", ((cpp_function_code)(&evenP)), NULL);
    defineFunctionObject("ODD?", "(DEFUN (ODD? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is an odd number.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGBITP 0 X) :CPP \"(x % 2)\" :JAVA \"((x % 2) == 1)\")))", ((cpp_function_code)(&oddP)), NULL);
    defineFunctionObject("MOD", "(DEFUN (MOD INTEGER) ((X INTEGER) (MODULO INTEGER)) :DOCUMENTATION \"Return the result of `x' mod `modulo'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:MOD X MODULO) :CPP \"(x % modulo)\" :JAVA \"(x % modulo)\")))", ((cpp_function_code)(&mod)), NULL);
    defineFunctionObject("CEILING", "(DEFUN (CEILING INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("FLOOR", "(DEFUN (FLOOR INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("ROUND", "(DEFUN (ROUND INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("EXP", "(DEFUN (EXP FLOAT) ((X FLOAT)) :PUBLIC? TRUE :NATIVE? TRUE :DOCUMENTATION \"The natural exponentiation function e^x.\")", NULL, NULL);
    defineFunctionObject("RANDOM", "(DEFUN (RANDOM INTEGER) ((N INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Generate a random integer in the interval [0..n-1].\n`n' must be <= 2^15.\")", ((cpp_function_code)(&random_)), NULL);
    defineFunctionObject("SEED-RANDOM-NUMBER-GENERATOR", "(DEFUN SEED-RANDOM-NUMBER-GENERATOR () :PUBLIC? TRUE :DOCUMENTATION \"Seeds the random number generator with the current time.\")", ((cpp_function_code)(&seedRandomNumberGenerator)), NULL);
    defineFunctionObject("SQRT", "(DEFUN (SQRT FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the square root of `n'.\" (RETURN (VERBATIM :COMMON-LISP (CL:SQRT N) :CPP \"sqrt(n)\" :JAVA \"Math.sqrt(n)\")))", ((cpp_function_code)(&sqrt_)), NULL);
    defineFunctionObject("COS", "(DEFUN (COS FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the cosine of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:COS N) :CPP \"cos(n)\" :JAVA \"Math.cos(n)\")))", ((cpp_function_code)(&cos_)), NULL);
    defineFunctionObject("SIN", "(DEFUN (SIN FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the sine of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:SIN N) :CPP \"sin(n)\" :JAVA \"Math.sin(n)\")))", ((cpp_function_code)(&sin_)), NULL);
    defineFunctionObject("TAN", "(DEFUN (TAN FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the tangent of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:TAN N) :CPP \"tan(n)\" :JAVA \"Math.tan(n)\")))", ((cpp_function_code)(&tan_)), NULL);
    defineFunctionObject("LOG", "(DEFUN (LOG FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the natural logarithm (base e) of `n'.\" (RETURN (VERBATIM :COMMON-LISP (CL:LOG N) :CPP \"log(n)\" :JAVA \"Math.log(n)\")))", ((cpp_function_code)(&log_)), NULL);
    defineFunctionObject("MIN", "(DEFUN (MIN INTEGER) ((X INTEGER) (Y INTEGER)) :DOCUMENTATION \"Return the minimum of `x' and `y'.  If either is NULL, return the other.\" :PUBLIC? TRUE)", ((cpp_function_code)(&min_)), NULL);
    defineFunctionObject("MAX", "(DEFUN (MAX INTEGER) ((X INTEGER) (Y INTEGER)) :DOCUMENTATION \"Return the maximum of `x' and `y'.  If either is NULL, return the other.\" :PUBLIC? TRUE)", ((cpp_function_code)(&max_)), NULL);
    defineMethodObject("(DEFMETHOD (ABS INTEGER) ((X INTEGER)) :DOCUMENTATION \"Return the absolute value of `x'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (CHOOSE (< X 0) (- 0 X) X)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (ABS FLOAT) ((X FLOAT)) :DOCUMENTATION \"Return the absolute value of `x'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (CHOOSE (< X 0.0) (- 0.0 X) X)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("CHARACTER-CODE", "(DEFUN (CHARACTER-CODE INTEGER) ((CHARACTER CHARACTER)) :DOCUMENTATION \"Return the 8-bit ASCII code of `character' as an integer.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-CODE CHARACTER) :CPP \"(int)(unsigned char) character\" :JAVA \"(int) character\")))", ((cpp_function_code)(&characterCode)), NULL);
    defineFunctionObject("CODE-CHARACTER", "(DEFUN (CODE-CHARACTER CHARACTER) ((CODE INTEGER)) :DOCUMENTATION \"Return the character encoded by `code' (0 <= `code' <= 255).\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CODE-CHAR CODE) :CPP \"(char) code\" :JAVA \"(char) code\")))", ((cpp_function_code)(&codeCharacter)), NULL);
    defineFunctionObject("CHARACTER-DOWNCASE", "(DEFUN (CHARACTER-DOWNCASE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the lowercase character for `c'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-DOWNCASE C) :CPP \"(isupper(c) ? c + 'a' - 'A' : c)\" :JAVA \"Character.toLowerCase(c)\")))", ((cpp_function_code)(&characterDowncase)), NULL);
    defineFunctionObject("CHARACTER-UPCASE", "(DEFUN (CHARACTER-UPCASE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the uppercase character for `c'.  If only the first character of\na sequence of characters is to be capitalized, character-capitalize should be used.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-UPCASE C) :CPP \"(islower(c) ? c + 'A' - 'a' : c)\" :JAVA \"Character.toUpperCase(c)\")))", ((cpp_function_code)(&characterUpcase)), NULL);
    defineFunctionObject("CHARACTER-CAPITALIZE", "(DEFUN (CHARACTER-CAPITALIZE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the capitalized character for `c'.  This is generally the same\nas the uppercase character, except for obscure non-English characters in Java.  It should\nbe used if only the first character of a sequence of characters is to be capitalized.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-UPCASE C) :CPP \"(islower(c) ? c + 'A' - 'a' : c)\" :JAVA \"Character.toTitleCase(c)\")))", ((cpp_function_code)(&characterCapitalize)), NULL);
    defineFunctionObject("STRING-EQL?", "(DEFUN (STRING-EQL? BOOLEAN) ((X STRING) (Y STRING)) :DOCUMENTATION \"Return true if `x' and `y' are equal strings including case for both null.\" :PUBLIC? TRUE)", ((cpp_function_code)(&stringEqlP)), NULL);
  }
}

void helpStartupPrimal3() {
  {
    defineFunctionObject("STRING-EQUAL?", "(DEFUN (STRING-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :DOCUMENTATION \"Return true if `x' and `y' are equal strings ignoring case for both null.\" :PUBLIC? TRUE)", ((cpp_function_code)(&stringEqualP)), NULL);
    defineFunctionObject("STRING-COMPARE", "(DEFUN (STRING-COMPARE INTEGER) ((X STRING) (Y STRING) (CASE-SENSITIVE? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Compare `x' and `y' lexicographically, and return -1, 0, \nor 1, depending on whether `x' is less than, equal, or greater than `y'.\nIf `case-sensitive?' is `true', then case does matter for the comparison\")", ((cpp_function_code)(&stringCompare)), NULL);
    defineFunctionObject("STRING<", "(DEFUN (STRING< BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically < `y', considering case.\")", ((cpp_function_code)(&stringL)), NULL);
    defineFunctionObject("STRING<=", "(DEFUN (STRING<= BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically <= `y', considering case..\")", ((cpp_function_code)(&stringLE)), NULL);
    defineFunctionObject("STRING>=", "(DEFUN (STRING>= BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically >= `y', considering case..\")", ((cpp_function_code)(&stringGE)), NULL);
    defineFunctionObject("STRING>", "(DEFUN (STRING> BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically > `y', considering case..\")", ((cpp_function_code)(&stringG)), NULL);
    defineFunctionObject("STRING-LESS?", "(DEFUN (STRING-LESS? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically < `y', ignoring case.\")", ((cpp_function_code)(&stringLessP)), NULL);
    defineFunctionObject("STRING-LESS-EQUAL?", "(DEFUN (STRING-LESS-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically <= `y', ignoring case.\")", ((cpp_function_code)(&stringLessEqualP)), NULL);
    defineFunctionObject("STRING-GREATER-EQUAL?", "(DEFUN (STRING-GREATER-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically >= `y', ignoring case.\")", ((cpp_function_code)(&stringGreaterEqualP)), NULL);
    defineFunctionObject("STRING-GREATER?", "(DEFUN (STRING-GREATER? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically > `y', ignoring case.\")", ((cpp_function_code)(&stringGreaterP)), NULL);
    defineFunctionObject("MAKE-STRING", "(DEFUN (MAKE-STRING STRING) ((SIZE INTEGER) (INITCHAR CHARACTER)) :DOCUMENTATION \"Return a new string filled with `size' `initchar's.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("MAKE-MUTABLE-STRING", "(DEFUN (MAKE-MUTABLE-STRING MUTABLE-STRING) ((SIZE INTEGER) (INITCHAR CHARACTER)) :DOCUMENTATION \"Return a new mutable string filled with `size' `initchar's.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:MAKE-STRING SIZE :INITIAL-ELEMENT INITCHAR) :CPP \"makeString(size, initchar)\" :JAVA \"#$(STELLAROOT).javalib.Native.makeMutableString(size, initchar)\")))", ((cpp_function_code)(&makeMutableString)), NULL);
    defineFunctionObject("MAKE-RAW-MUTABLE-STRING", "(DEFUN (MAKE-RAW-MUTABLE-STRING MUTABLE-STRING) ((SIZE INTEGER)) :DOCUMENTATION \"Return a new uninitialized mutable string of `size'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (LET ((S MUTABLE-STRING NULL)) (VERBATIM :COMMON-LISP (SETQ S (CL:MAKE-STRING SIZE)) :CPP \"s = new (GC) char[size]\" :JAVA \"s = new StringBuffer(size); s.setLength(size)\") (RETURN S)))", ((cpp_function_code)(&makeRawMutableString)), NULL);
    defineFunctionObject("STRING-CONCATENATE", "(DEFUN (STRING-CONCATENATE STRING) ((STRING1 STRING) (STRING2 STRING)) :DOCUMENTATION \"Return a new string representing the concatenation of\n`string1' and `string2'.\" :NATIVE? TRUE)", NULL, NULL);
    defineMethodObject("(DEFMETHOD (CONCATENATE STRING) ((STRING1 STRING) (STRING2 STRING) |&REST| (OTHERSTRINGS STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a new string representing the concatenation\nof `string1', `string2', and `otherStrings'.  The two mandatory parameters\nallow us to optimize the common binary case by not relying on the somewhat\nless efficient variable arguments mechanism.\")", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("STRING-UPCASE", "(DEFUN (STRING-UPCASE STRING) ((STRING STRING)) :DOCUMENTATION \"Return an upper-case copy of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("STRING-DOWNCASE", "(DEFUN (STRING-DOWNCASE STRING) ((STRING STRING)) :DOCUMENTATION \"Return a lower-case copy of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("STRING-CAPITALIZE", "(DEFUN (STRING-CAPITALIZE STRING) ((STRING STRING)) :DOCUMENTATION \"Return a capitalized version of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineMethodObject("(DEFMETHOD (COPY STRING) ((STRING STRING)) :DOCUMENTATION \"Return a copy of `string'.\" :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (SUBSTITUTE MUTABLE-STRING) ((SELF MUTABLE-STRING) (NEWCHAR CHARACTER) (OLDCHAR CHARACTER)) :DOCUMENTATION \"Substitute all occurences of `oldChar' with `newChar'\nin the string `self'.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (SUBSTITUTE STRING) ((SELF STRING) (NEWCHAR CHARACTER) (OLDCHAR CHARACTER)) :DOCUMENTATION \"Substitute all occurences of `oldChar' with `newChar'\nin the string `self'.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("REPLACE-SUBSTRINGS", "(DEFUN (REPLACE-SUBSTRINGS STRING) ((STRING STRING) (NEW STRING) (OLD STRING)) :DOCUMENTATION \"Replace all occurrences of `old' in `string' with `new'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&replaceSubstrings)), NULL);
    defineFunctionObject("INSERT-STRING", "(DEFUN (INSERT-STRING INTEGER) ((SOURCE STRING) (START INTEGER) (END INTEGER) (TARGET MUTABLE-STRING) (TARGET-INDEX INTEGER) (CASE-CONVERSION KEYWORD)) :DOCUMENTATION \"Inserts characters from `source' begining at `start' and\nending at `end' into `target' starting at `target-index'.  If `end' is `null',\nthen the entire length of the string is used. The copy of characters is affected\nby the `case-conversion' keyword which should be one of\n   :UPCASE :DOWNCASE :CAPITALIZE :PRESERVE.\n\nThe final value of target-index is returned.\")", ((cpp_function_code)(&insertString)), NULL);
    defineFunctionObject("INTEGER-TO-STRING", "(DEFUN (INTEGER-TO-STRING STRING) ((I INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("FLOAT-TO-STRING", "(DEFUN (FLOAT-TO-STRING STRING) ((F FLOAT)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("CHARACTER-TO-STRING", "(DEFUN (CHARACTER-TO-STRING STRING) ((C CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (MAKE-STRING 1 C)))", ((cpp_function_code)(&characterToString)), NULL);
    defineFunctionObject("FORMAT-FLOAT", "(DEFUN (FORMAT-FLOAT STRING) ((F FLOAT) (NDECIMALS INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("STRING-TO-INTEGER", "(DEFUN (STRING-TO-INTEGER INTEGER) ((STRING STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("STRING-TO-FLOAT", "(DEFUN (STRING-TO-FLOAT FLOAT) ((STRING STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("FORMAT-WITH-PADDING", "(DEFUN (FORMAT-WITH-PADDING STRING) ((INPUT STRING) (LENGTH INTEGER) (PADCHAR CHARACTER) (ALIGN KEYWORD) (TRUNCATE? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Formats `input' to be (at least) `length' long, using `padchar' to\nfill if necessary.  `align' must be one of :LEFT, :RIGHT, :CENTER and will control\nhow `input' will be justified in the resulting string.  If `truncate?' is `true', then\nthen an overlength string will be truncated, using the opposite of `align' to pick\nthe truncation direction.\")", ((cpp_function_code)(&formatWithPadding)), NULL);
    defineMethodObject("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF STRING) (CHAR CHARACTER)) :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH CHARACTER) ((SELF STRING) (POSITION INTEGER)) :DOCUMENTATION \"Return the character in `self' at `position'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) :CPP \"self[position]\" :JAVA \"self.charAt(position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH CHARACTER) ((SELF MUTABLE-STRING) (POSITION INTEGER)) :DOCUMENTATION \"Return the character in `self' at `position'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) :CPP \"self[position]\" :JAVA \"self.charAt(position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH-SETTER CHARACTER) ((SELF MUTABLE-STRING) (CH CHARACTER) (POSITION INTEGER)) :DOCUMENTATION \"Set the character in `self' at `position' to `ch'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (SETF (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) (CL:THE CL:CHARACTER CH)) :CPP \"self[position] = ch\" :JAVA \"#$(STELLAROOT).javalib.Native.mutableString_nthSetter(self, ch, position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (REST STRING) ((SELF STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LENGTH INTEGER) ((SELF STRING)) :DOCUMENTATION \"Return the length of the string `self'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LENGTH SELF) :CPP \"strlen(self)\" :JAVA \"self.length()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LENGTH INTEGER) ((SELF MUTABLE-STRING)) :DOCUMENTATION \"Return the length of the string `self'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LENGTH SELF) :CPP \"strlen(self)\" :JAVA \"self.length()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (POSITION INTEGER) ((STRING STRING) (CHARACTER CHARACTER) (START INTEGER)) :DOCUMENTATION \"Return the position of `character' within `string' (counting\nfrom zero); or return NULL if `character' does not occur within `string'.\nIf `start' was supplied as non-NULL, only consider the substring starting\nat `start', however, the returned position will always be relative to the\nentire string.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("STRING-SEARCH", "(DEFUN (STRING-SEARCH INTEGER) ((STRING STRING) (SUBSTRING STRING) (START INTEGER)) :DOCUMENTATION \"Return start position of the left-most occurrence of\n`substring' in `string', beginning from `start'.  Return NULL if it is not\na substring.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineMethodObject("(DEFMETHOD (SUBSEQUENCE STRING) ((STRING STRING) (START INTEGER) (END INTEGER)) :DOCUMENTATION \"Return a substring of `string' beginning at position `start'\nand ending up to but not including position `end', counting from zero.  An\n`end' value of NULL stands for the rest of the string.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("STRINGIFY", "(DEFUN (STRINGIFY STRING) ((EXPRESSION OBJECT)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("UNSTRINGIFY", "(DEFUN (UNSTRINGIFY OBJECT) ((STRING STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (READ-S-EXPRESSION-FROM-STRING STRING)))", ((cpp_function_code)(&unstringify)), NULL);
    defineMethodObject("(DEFMETHOD FREE-HASH-TABLE-VALUES ((SELF ABSTRACT-HASH-TABLE)) :DOCUMENTATION \"Call free on each value in the hash table `self'.\")", ((cpp_method_code)(&AbstractHashTable::freeHashTableValues)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF ABSTRACT-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&AbstractHashTable::initializeHashTable)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&HashTable::lookup)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&HashTable::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD REMOVE-AT ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&HashTable::removeAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF INTEGER-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&IntegerHashTable::initializeHashTable)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF INTEGER-HASH-TABLE) (KEY INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&IntegerHashTable::lookup)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF INTEGER-HASH-TABLE) (KEY INTEGER) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&IntegerHashTable::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD REMOVE-AT ((SELF INTEGER-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))))", ((cpp_method_code)(&IntegerHashTable::removeAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF FLOAT-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&FloatHashTable::initializeHashTable)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF FLOAT-HASH-TABLE) (KEY FLOAT)) :PUBLIC? TRUE)", ((cpp_method_code)(&FloatHashTable::lookup)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF FLOAT-HASH-TABLE) (KEY FLOAT) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&FloatHashTable::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD REMOVE-AT ((SELF FLOAT-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))))", ((cpp_method_code)(&FloatHashTable::removeAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF STRING-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&StringHashTable::initializeHashTable)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STRING-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringHashTable::lookup)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF STRING-HASH-TABLE) (KEY STRING) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringHashTable::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD REMOVE-AT ((SELF STRING-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringHashTable::removeAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF STRING-TO-INTEGER-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&StringToIntegerHashTable::initializeHashTable)), ((cpp_method_code)(NULL)));
  }
}

void helpStartupPrimal4() {
  {
    defineMethodObject("(DEFMETHOD (LOOKUP INTEGER) ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringToIntegerHashTable::lookup)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD INSERT-AT ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING) (VALUE INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringToIntegerHashTable::insertAt)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD REMOVE-AT ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&StringToIntegerHashTable::removeAt)), ((cpp_method_code)(NULL)));
    defineFunctionObject("OBJECT-HASH-CODE", "(DEFUN (OBJECT-HASH-CODE INTEGER) ((SELF OBJECT)) :DOCUMENTATION \"Return a hash code for `self'.  Two objects that are `eq?'\nare guaranteed to generate the same hash code.  Two objects that are not `eq?'\ndo not necessarily generate different hash codes.  Similar to `hash-code' but\nalways hashes on the address of `self' even if it is a wrapper.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP \"(STELLA::%%OBJECT-HASH-CODE SELF)\" :CPP \"(size_t)self\" :JAVA \"self.hashCode()\")))", ((cpp_function_code)(&objectHashCode)), NULL);
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF OBJECT)) :DOCUMENTATION \"Return a hash code for `self'.  Two objects that are `eql?'\nare guaranteed to generate the same hash code.  Two objects that are not `eql?'\ndo not necessarily generate different hash codes.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STANDARD-OBJECT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (OBJECT-HASH-CODE SELF)))", ((cpp_method_code)(&StandardObject::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Wrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STRING-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&StringWrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF INTEGER-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&IntegerWrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF FLOAT-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&FloatWrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF CHARACTER-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&CharacterWrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF BOOLEAN-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (CHOOSE (WRAPPER-VALUE SELF) 27333705 31891526)))", ((cpp_method_code)(&BooleanWrapper::hashCode)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SXHASH SELF) :JAVA \"self.hashCode()\" :OTHERWISE (HASH-STRING SELF 0))))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN SELF))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SXHASH SELF) :CPP \"(size_t)self\" :JAVA \"new Float(self).hashCode()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (HASH-CODE INTEGER) ((SELF CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (LOGXOR (AREF *HASH-BYTE-RANDOM-TABLE* (CHARACTER-CODE SELF)) 33485726)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("HASH-STRING", "(DEFUN (HASH-STRING INTEGER) ((STRING STRING) (SEEDCODE INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE :DOCUMENTATION \"Generate a hash-code for `string' and return it.\nTwo strings that are equal but not eq will generate the same code.\nThe hash-code is based on `seedCode' which usually will be 0.  However,\n`seedCode' can also be used to supply the result of a previous hash\noperation to achieve hashing on sequences of strings without actually\nhaving to concatenate them.\" (LET ((CODE SEEDCODE) (CURSOR (LENGTH STRING))) (IF (= CURSOR 0) (RETURN 4303803) (-- CURSOR)) (LOOP (SETQ CODE (LOGXOR CODE (AREF *HASH-BYTE-RANDOM-TABLE* (CHARACTER-CODE (NTH STRING CURSOR))))) (IF (= CURSOR 0) (BREAK) (-- CURSOR)) (IF (= (LOGAND CODE 1) 0) (SETQ CODE (SHIFT-RIGHT CODE 1)) (SETQ CODE (LOGOR (SHIFT-RIGHT CODE 1) 67108864)))) (RETURN CODE)))", ((cpp_function_code)(&hashString)), NULL);
    defineFunctionObject("PICK-HASH-TABLE-SIZE-PRIME", "(DEFUN (PICK-HASH-TABLE-SIZE-PRIME INTEGER) ((MINSIZE INTEGER)) :DOCUMENTATION \"Return a hash table prime of at least `minSize'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&pickHashTableSizePrime)), NULL);
    defineMethodObject("(DEFMETHOD INITIALIZE-VECTOR ((SELF VECTOR)))", ((cpp_method_code)(&Vector::initializeVector)), ((cpp_method_code)(NULL)));
    defineFunctionObject("RESIZE-VECTOR", "(DEFUN RESIZE-VECTOR ((SELF VECTOR) (SIZE INTEGER)) :DOCUMENTATION \"Change the size of `self' to `size'.  If `size' is smaller\nthan the current size of `self' the vector will be truncated.  Otherwise, \nthe internal array of `self' will be grown to `size' and unused elements\nwill be initialized to NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&resizeVector)), NULL);
    defineMethodObject("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF NATIVE-VECTOR) (POSITION INTEGER)) :DOCUMENTATION \"Return the element in `self' at `position'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF SELF POSITION) :CPP \"self[position]\" :JAVA \"self[position]\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineMethodObject("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF NATIVE-VECTOR) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :DOCUMENTATION \"Set the element in `self' at `position' to `value'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SETF (CL:AREF SELF POSITION) VALUE) :CPP \"self[position] = value\" :JAVA \"self[position] = value\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    defineFunctionObject("UNMAKE", "(DEFUN UNMAKE ((SELF OBJECT)))", ((cpp_function_code)(&unmake)), NULL);
    defineFunctionObject("PROBE-FILE?", "(DEFUN (PROBE-FILE? BOOLEAN) ((FILENAME FILE-NAME)) :DOCUMENTATION \"Return `true' if file `fileName' exists.  Note that this does\nnot necessarily mean that the file can also be read.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("FILE-WRITE-DATE", "(DEFUN (FILE-WRITE-DATE CALENDAR-DATE) ((FILENAME FILE-NAME)) :DOCUMENTATION \"Return the time at which file `fileName' was last modified or\nNULL if that cannot be determined.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("FILE-LENGTH", "(DEFUN (FILE-LENGTH INTEGER) ((FILENAME FILE-NAME)) :DOCUMENTATION \"Return the length of file `fileName' in bytes or NULL if that\ncannot be determined.  Note that this will currently overrun for files that\nare longer than what can be represented by a STELLA integer.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("DELETE-FILE", "(DEFUN DELETE-FILE ((FILENAME FILE-NAME)) :DOCUMENTATION \"Delete the file `fileName'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("COPY-FILE", "(DEFUN COPY-FILE ((FROMFILE FILE-NAME) (TOFILE FILE-NAME)) :DOCUMENTATION \"Copy file `fromFile' to file `toFile', clobbering\nany data already in `toFile'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&copyFile)), NULL);
    defineFunctionObject("LOGNOT", "(DEFUN (LOGNOT INTEGER) ((ARG INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGNOT ARG) :CPP \"(~ arg)\" :JAVA \"(~ arg)\")))", ((cpp_function_code)(&lognot)), NULL);
    defineFunctionObject("LOGAND", "(DEFUN (LOGAND INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGAND ARG1 ARG2) :CPP \"(arg1 & arg2)\" :JAVA \"(arg1 & arg2)\")))", ((cpp_function_code)(&logand)), NULL);
    defineFunctionObject("LOGOR", "(DEFUN (LOGOR INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGIOR ARG1 ARG2) :CPP \"(arg1 | arg2)\" :JAVA \"(arg1 | arg2)\")))", ((cpp_function_code)(&logor)), NULL);
    defineFunctionObject("LOGXOR", "(DEFUN (LOGXOR INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGXOR ARG1 ARG2) :CPP \"(arg1 ^ arg2)\" :JAVA \"(arg1 ^ arg2)\")))", ((cpp_function_code)(&logxor)), NULL);
    defineFunctionObject("INTEGER-LENGTH", "(DEFUN (INTEGER-LENGTH INTEGER) ((ARG INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    defineFunctionObject("SHIFT-LEFT", "(DEFUN (SHIFT-LEFT INTEGER) ((ARG INTEGER) (COUNT INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:ASH ARG COUNT) :CPP \"(arg << count)\" :JAVA \"(arg << count)\")))", ((cpp_function_code)(&shiftLeft)), NULL);
    defineFunctionObject("SHIFT-RIGHT", "(DEFUN (SHIFT-RIGHT INTEGER) ((ARG INTEGER) (COUNT INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:ASH ARG (CL:THE CL:FIXNUM (CL:- COUNT))) :CPP \"(arg >> count)\" :JAVA \"(arg >> count)\")))", ((cpp_function_code)(&shiftRight)), NULL);
    defineFunctionObject("STELLIFY", "(DEFUN (STELLIFY OBJECT) ((SELF OBJECT)) :DOCUMENTATION \"Convert a Lisp object into a STELLA object.\" :PUBLIC? TRUE)", ((cpp_function_code)(&stellify)), NULL);
    defineFunctionObject("STELLA-OBJECT?", "(DEFUN (STELLA-OBJECT? BOOLEAN) ((SELF OBJECT)) :DOCUMENTATION \"Return TRUE if `self' is a member of the STELLA class `OBJECT'.\")", ((cpp_function_code)(&stellaObjectP)), NULL);
    defineFunctionObject("RUNNING-AS-LISP?", "(DEFUN (RUNNING-AS-LISP? BOOLEAN) () :DOCUMENTATION \"Return TRUE if the executable code is a Common Lisp application.\")", ((cpp_function_code)(&runningAsLispP)), NULL);
    defineFunctionObject("GET-TICKTOCK", "(DEFUN (GET-TICKTOCK TICKTOCK) () :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("TICKTOCK-DIFFERENCE", "(DEFUN (TICKTOCK-DIFFERENCE FLOAT) ((T1 TICKTOCK) (T2 TICKTOCK)) :DOCUMENTATION \"The difference in two TICKTOCK time values in seconds where\n`t1' is the earlier time.  The resolution is implementation dependent but will\nnormally be some fractional value of a second.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("TICKTOCK-RESOLUTION", "(DEFUN (TICKTOCK-RESOLUTION FLOAT) () :DOCUMENTATION \"The minimum theoretically detectable resolution of the\ndifference in two TICKTOCK time values in seconds.  This\nresolution is implementation dependent.  It may also not\nbe realizable in practice, since the timing grain size may\nbe larger than this resolution.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    defineFunctionObject("SLEEP", "(DEFUN SLEEP ((SECONDS FLOAT)) :DOCUMENTATION \"The program will sleep for the indicated number of seconds.\nFractional values are allowed, but the results are implementation dependent:\nCommon Lisp uses the fractions natively, Java with a resolution of 0.001,\nand C++ can only use integral values.\")", ((cpp_function_code)(&sleep_)), NULL);
    defineFunctionObject("EXCEPTION-MESSAGE", "(DEFUN (EXCEPTION-MESSAGE STRING) ((E NATIVE-EXCEPTION)) :PUBLIC? TRUE :DOCUMENTATION \"Accesses the error message of the exception `e'.\")", ((cpp_function_code)(&exceptionMessage)), NULL);
    defineFunctionObject("PRINT-EXCEPTION-CONTEXT", "(DEFUN PRINT-EXCEPTION-CONTEXT ((E NATIVE-EXCEPTION) (STREAM OUTPUT-STREAM)) :PUBLIC? TRUE :DOCUMENTATION \"Prints a system dependent information about the context of the specified\nexception.  For example, in Java it prints a stack trace.  In Lisp, it is vendor dependent.\")", ((cpp_function_code)(&printExceptionContext)), NULL);
    defineFunctionObject("MAKE-PROCESS-LOCK", "(DEFUN (MAKE-PROCESS-LOCK PROCESS-LOCK-OBJECT) ())", ((cpp_function_code)(&makeProcessLock)), NULL);
    defineFunctionObject("STARTUP-PRIMAL", "(DEFUN STARTUP-PRIMAL () :PUBLIC? TRUE)", ((cpp_function_code)(&startupPrimal)), NULL);
    { MethodSlot* function = lookupFunction(SYM_PRIMAL_STELLA_STARTUP_PRIMAL);

      setDynamicSlotValue(function->dynamicSlots, SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Primal"), NULL_STRING_WRAPPER);
    }
  }
}

void startupPrimal() {
  if (currentStartupTimePhaseP(0)) {
    
#ifndef HAVE_SGI_HASH_MAPS
  oUSE_STELLA_HASH_TABLESpo = TRUE;
#endif
;
    oHASH_BYTE_RANDOM_TABLEo = new (GC)(int[256]);
    { int i = NULL_INTEGER;
      int iter007 = 0;
      int upperBound008 = 255;

      while (iter007 <= upperBound008) {
        i = iter007;
        iter007 = iter007 + 1;
        oHASH_BYTE_RANDOM_TABLEo[i] = 0;
      }
    }
    { int bit = NULL_INTEGER;
      int iter009 = 0;
      int upperBound010 = 25;

      while (iter009 <= upperBound010) {
        bit = iter009;
        iter009 = iter009 + 1;
        { int i = NULL_INTEGER;
          int iter011 = 0;
          int upperBound012 = 255;

          while (iter011 <= upperBound012) {
            i = iter011;
            iter011 = iter011 + 1;
            oHASH_BYTE_RANDOM_TABLEo[i] = ((oHASH_BYTE_RANDOM_TABLEo[i]) | (((random_(2)) << bit)));
          }
        }
      }
    }
    oHASH_TABLE_SIZE_PRIMESo = list(25, wrapInteger(29), wrapInteger(53), wrapInteger(97), wrapInteger(193), wrapInteger(389), wrapInteger(769), wrapInteger(1543), wrapInteger(3079), wrapInteger(6151), wrapInteger(12289), wrapInteger(24593), wrapInteger(49157), wrapInteger(98317), wrapInteger(196613), wrapInteger(393241), wrapInteger(786433), wrapInteger(1572869), wrapInteger(3145739), wrapInteger(6291469), wrapInteger(12582917), wrapInteger(25165843), wrapInteger(50331653), wrapInteger(100663319), wrapInteger(201326611), wrapInteger(402653189));
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      helpStartupPrimal1();
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      helpStartupPrimal2();
      helpStartupPrimal3();
      helpStartupPrimal4();
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
    if (currentStartupTimePhaseP(9)) {
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT TRUE BOOLEAN (VERBATIM :COMMON-LISP CL:T :CPP \"1\" :JAVA \"true\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT FALSE BOOLEAN (VERBATIM :COMMON-LISP CL:NIL :CPP \"0\" :JAVA \"false\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT PI FLOAT (VERBATIM :COMMON-LISP \"(cl:float CL:PI 0.0d0)\" :CPP \"3.141592653589793\" :JAVA \"java.lang.Math.PI\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(defconstant NULL UNKNOWN NULL)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-INTEGER INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (int) / sizeof (char) * 8 - 1)\" :JAVA \"Integer.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-SHORT-INTEGER SHORT-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (short int) / sizeof (char) * 8 - 1)\" :JAVA \"Short.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-LONG-INTEGER LONG-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (long int) / sizeof (char) * 8 - 1)\" :JAVA \"Long.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-UNSIGNED-SHORT-INTEGER UNSIGNED-SHORT-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"(unsigned short int) -1\" :JAVA \"Short.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-UNSIGNED-LONG-INTEGER UNSIGNED-LONG-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"(unsigned long int) -1\" :JAVA \"Long.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-FLOAT FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-DOUBLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Double.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-SINGLE-FLOAT SINGLE-FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-SINGLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Float.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-DOUBLE-FLOAT DOUBLE-FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-DOUBLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Double.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-CHARACTER CHARACTER (VERBATIM :COMMON-LISP (CL:CODE-CHAR 0) :CPP \"'\\\\0'\" :JAVA \"Character.MIN_VALUE\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-BYTE BYTE (VERBATIM :COMMON-LISP \"255\" :CPP \"255\" :JAVA \"-128\") :PUBLIC? TRUE)");
      defineStellaGlobalVariableFromStringifiedSource("(DEFCONSTANT NULL-OCTET OCTET (VERBATIM :COMMON-LISP \"255\" :CPP \"255\" :JAVA \"-128\") :PUBLIC? TRUE)");
      registerNativeName(SYM_PRIMAL_STELLA_FLOOR, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_RANDOM, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_SQRT, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_COS, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_SIN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_TAN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_LOG, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_MIN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      registerNativeName(SYM_PRIMAL_STELLA_MAX, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      defineStellaGlobalVariableFromStringifiedSource("(DEFSPECIAL *TRANSIENTOBJECTS?* BOOLEAN FALSE :PUBLIC? TRUE)");
      registerNativeName(SYM_PRIMAL_STELLA_HASH_CODE, KWD_PRIMAL_JAVA, KWD_PRIMAL_FUNCTION);
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *HASH-BYTE-RANDOM-TABLE* (ARRAY (256) OF INTEGER) NULL :DOCUMENTATION \"A table of 256 26-bit random numbers that can be used to\nhash sequences of bytes.  Each bit-column in the table has an approximately\neven number of 0's and 1's.\")");
      defineStellaGlobalVariableFromStringifiedSource("(DEFGLOBAL *HASH-TABLE-SIZE-PRIMES* (LIST OF INTEGER-WRAPPER) NULL :DOCUMENTATION \"List of prime numbers approximately growing by a factor of 2\nthat are suitable to be used as hash table sizes.\" :PUBLIC? TRUE)");
      registerNativeName(SYM_PRIMAL_STELLA_SLEEP, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
    }
  }
}

Symbol* SYM_PRIMAL_STELLA_FLOOR = NULL;

Keyword* KWD_PRIMAL_CPP = NULL;

Keyword* KWD_PRIMAL_FUNCTION = NULL;

Symbol* SYM_PRIMAL_STELLA_RANDOM = NULL;

Symbol* SYM_PRIMAL_STELLA_SQRT = NULL;

Symbol* SYM_PRIMAL_STELLA_COS = NULL;

Symbol* SYM_PRIMAL_STELLA_SIN = NULL;

Symbol* SYM_PRIMAL_STELLA_TAN = NULL;

Symbol* SYM_PRIMAL_STELLA_LOG = NULL;

Symbol* SYM_PRIMAL_STELLA_MIN = NULL;

Symbol* SYM_PRIMAL_STELLA_MAX = NULL;

Keyword* KWD_PRIMAL_UPCASE = NULL;

Keyword* KWD_PRIMAL_DOWNCASE = NULL;

Keyword* KWD_PRIMAL_CAPITALIZE = NULL;

Keyword* KWD_PRIMAL_PRESERVE = NULL;

Keyword* KWD_PRIMAL_LEFT = NULL;

Keyword* KWD_PRIMAL_RIGHT = NULL;

Keyword* KWD_PRIMAL_CENTER = NULL;

Symbol* SYM_PRIMAL_STELLA_HASH_CODE = NULL;

Keyword* KWD_PRIMAL_JAVA = NULL;

Symbol* SYM_PRIMAL_STELLA_SLEEP = NULL;

Symbol* SYM_PRIMAL_STELLA_STARTUP_PRIMAL = NULL;

Symbol* SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
